# src\file_conversor\backend\hash_backend.py

import hashlib

from typing import Any, Callable, Sequence

from pathlib import Path

from rich import print

# user-provided imports
from file_conversor.backend.abstract_backend import AbstractBackend

from file_conversor.config import Environment, Configuration, Log
from file_conversor.config.locale import get_translation

# get app config
CONFIG = Configuration.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


class HashCheckFailed(RuntimeError):
    def __init__(self, filename: str | Path, expected: str, actual: str) -> None:
        super().__init__()
        self._filename = str(filename)
        self._expected = expected
        self._actual = actual

    def __str__(self) -> str:
        return f"'{self._filename}' - Expected: {self._expected} - Actual: {self._actual}"

    def __repr__(self) -> str:
        return f"{type(self)}({self.__str__()})"


class HashBackend(AbstractBackend):
    SUPPORTED_IN_FORMATS = {
        "md5": {},
        "sha1": {},
        "sha256": {},
        "sha384": {},
        "sha512": {},
        "sha3_256": {},
        "sha3_384": {},
        "sha3_512": {},
    }
    SUPPORTED_OUT_FORMATS = {
        "md5": {
            "gen_algo": hashlib.md5
        },
        "sha1": {
            "gen_algo": hashlib.sha1
        },
        "sha256": {
            "gen_algo": hashlib.sha256
        },
        "sha384": {
            "gen_algo": hashlib.sha384
        },
        "sha512": {
            "gen_algo": hashlib.sha512
        },
        "sha3_256": {
            "gen_algo": hashlib.sha3_256
        },
        "sha3_384": {
            "gen_algo": hashlib.sha3_384
        },
        "sha3_512": {
            "gen_algo": hashlib.sha3_512
        },
    }
    EXTERNAL_DEPENDENCIES: set[str] = set()

    def __init__(
        self,
        verbose: bool = False,
    ):
        """
        Initialize the Batch backend.
        """
        super().__init__()
        self._verbose = verbose

    def _get_hash(
            self,
            input_file: str | Path,
            algorithm: str,
    ):
        input_file = Path(input_file)
        if not input_file.exists():
            raise FileNotFoundError(f"'{input_file}' not found")
        data = input_file.read_bytes()
        algorithm_callback = self.SUPPORTED_OUT_FORMATS[algorithm]["gen_algo"]
        return algorithm_callback(data).hexdigest()

    def generate(
            self,
            input_files: Sequence[str | Path],
            output_file: str | Path,
            progress_callback: Callable[[float], Any] | None = None,
    ):
        """
        Generates file hash

        :param input_files: Input files
        :param output_file: Output file
        :param progress_callback: Progress callback (0-100). Defaults to None.
        """
        res = ""
        output_file = Path(output_file)
        output_file = output_file.with_suffix(output_file.suffix.lower())

        out_ext = output_file.suffix[1:]

        input_len = len(input_files)
        for idx, in_file in enumerate(input_files, start=1):
            input_file = Path(in_file)
            digest = self._get_hash(input_file, out_ext)
            res += f"{digest}  {input_file.name}\n"
            if progress_callback:
                progress_callback(100.0 * (float(idx) / input_len))

        output_file.write_text(res, encoding="utf-8")

    def check(
            self,
            input_file: str | Path,
            progress_callback: Callable[[float], Any] | None = None,
    ):
        """
        Checks file hash

        :param input_files: Input files
        :param progress_callback: Progress callback (0-100). Defaults to None.

        :raises HashCheckFailed: if hash is not correct
        """
        input_file = Path(input_file)
        in_ext = input_file.suffix[1:]
        if not input_file.exists():
            raise FileNotFoundError(f"'{input_file}' not found")

        lines = input_file.read_text().splitlines()
        for idx, line in enumerate(lines, start=1):
            digest, filename = line.strip().split()
            filename = input_file.parent / filename
            actual = self._get_hash(filename, in_ext)
            if actual != digest:
                logger.error(rf"'{filename}': [bold red]FAILED[/]")
                raise HashCheckFailed(filename, expected=digest, actual=actual)
            logger.info(rf"'{filename}': [bold green]OK[/]")

            progress = 100.0 * (float(idx) / len(lines))
            if progress_callback:
                progress_callback(progress)


__all__ = [
    "HashBackend",
]



# src/file_conversor/backend/gui/pdf/convert.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.pdf import PyMuPDFBackend, PDF2DOCXBackend

from file_conversor.backend.gui._dom_page import home_nav_item
from file_conversor.backend.gui.pdf._dom_page import pdf_convert_nav_item, pdf_index_nav_item

from file_conversor.utils.bulma_utils import FileFormatField, InputFilesField, OutputDirField, ImageDPIField, PDFPasswordField
from file_conversor.utils.dominate_bulma import PageForm

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def PagePDFConvert():
    return PageForm(
        InputFilesField(
            "pdf",
            description=_("PDF files"),
        ),
        FileFormatField(
            *[
                (f, f.upper())
                for f in {
                    **PDF2DOCXBackend.SUPPORTED_OUT_FORMATS,
                    **PyMuPDFBackend.SUPPORTED_OUT_FORMATS
                }
            ],
            current_value="docx",
        ),
        ImageDPIField(),
        PDFPasswordField(),
        OutputDirField(),
        api_endpoint=f"{url_for('api_pdf_convert')}",
        nav_items=[
            home_nav_item(),
            pdf_index_nav_item(),
            pdf_convert_nav_item(active=True),
        ],
        _title=f"{_('PDF Convert')} - File Conversor",
    )


def pdf_convert():
    return render_template_string(str(
        PagePDFConvert()
    ))




# src\file_conversor\cli\image\to_pdf_cmd.py

import typer

from pathlib import Path
from typing import Annotated, Any, Callable, List

from rich import print

# user-provided modules
from file_conversor.backend.image import Img2PDFBackend

from file_conversor.cli.image._typer import CONVERSION_PANEL as RICH_HELP_PANEL
from file_conversor.cli.image._typer import COMMAND_NAME, TO_PDF_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.typer_utils import DPIOption, InputFilesArgument, OutputFileOption
from file_conversor.utils.validators import check_is_bool_or_none, check_path_exists, check_valid_options

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = Img2PDFBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    # IMG2PDF commands
    for ext in Img2PDFBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="to_pdf",
                description="To PDF",
                command=f'cmd.exe /c "{Environment.get_executable()} "{COMMAND_NAME}" "{TO_PDF_NAME}" "%1""',
                icon=str(icons_folder_path / "pdf.ico"),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_image_to_pdf_cmd(
    input_files: List[Path],
    dpi: int,
    fit: str,
    page_size: str | None,
    set_metadata: bool,
    output_file: Path | None,
    progress_callback: Callable[[float], Any] = lambda p: p,
):
    output_file = output_file if output_file else Path() / CommandManager.get_output_file(input_files[0], suffix=".pdf")
    if not STATE["overwrite-output"]:
        check_path_exists(output_file, exists=False)

    img2pdf_backend = Img2PDFBackend(verbose=STATE['verbose'])
    # display current progress
    with ProgressManager() as progress_mgr:
        page_sz: tuple | None = None
        if page_size in Img2PDFBackend.PAGE_LAYOUT:
            page_sz = Img2PDFBackend.PAGE_LAYOUT[page_size]
        elif page_size:
            page_sz = tuple(page_size)

        img2pdf_backend.to_pdf(
            input_files=input_files,
            output_file=output_file,
            dpi=dpi,
            image_fit=Img2PDFBackend.FIT_MODES[fit],
            page_size=page_sz,
            include_metadata=set_metadata,
        )
        progress_callback(progress_mgr.complete_step())
    logger.info(f"{_('PDF generation')}: [green bold]{_('SUCCESS')}[/]")


@typer_cmd.command(
    name=TO_PDF_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Convert a list of image files to one PDF file, one image per page.')}

        Fit = {_('Valid only if ``--page-size`` is defined. Otherwise, PDF size = figure size.')}

        - 'into': {_('Figure adjusted to fit in PDF size')}.

        - 'fill': {_('Figure adjusted to fit in PDF size')}, {_('without any empty borders (cut figure if needed)')}.
    """,
    epilog=f"""
        **{_('Examples')}:**

        - `file_conversor {COMMAND_NAME} {TO_PDF_NAME} input_file.jpg -of output_file.pdf --dpi 96`

        - `file_conversor {COMMAND_NAME} {TO_PDF_NAME} input_file1.bmp input_file2.png -of output_file.pdf`

        - `file_conversor {COMMAND_NAME} {TO_PDF_NAME} input_file.jpg -of output_file.pdf -ps a4_landscape`

        - `file_conversor {COMMAND_NAME} {TO_PDF_NAME} input_file1.bmp input_file2.png -of output_file.pdf --page-size (21.00,29.70)`
    """)
def to_pdf(
    input_files: Annotated[List[Path], InputFilesArgument(Img2PDFBackend)],
    dpi: Annotated[int, DPIOption()] = CONFIG["image-dpi"],
    fit: Annotated[str, typer.Option("--fit", "-f",
                                     help=f"{_("Image fit. Valid only if ``--page-size`` is defined. Valid values are")} {", ".join(Img2PDFBackend.FIT_MODES)}. {_("Defaults to")} {CONFIG["image-fit"]}",
                                     callback=lambda x: check_valid_options(x.lower(), Img2PDFBackend.FIT_MODES),
                                     )] = CONFIG["image-fit"],

    page_size: Annotated[str | None, typer.Option("--page-size", "-ps",
                                                  help=f"{_("Page size. Format '(width, height)'. Other valid values are:")} {", ".join(Img2PDFBackend.PAGE_LAYOUT)}. {_("Defaults to None (PDF size = image size).")}",
                                                  callback=lambda x: check_valid_options(x.lower() if x else None, Img2PDFBackend.PAGE_LAYOUT),
                                                  )] = CONFIG["image-page-size"],

    set_metadata: Annotated[bool, typer.Option("--set-metadata", "-sm",
                                               help=_("Set PDF metadata. Defaults to False (do not set creator, producer, modification date, etc)."),
                                               callback=check_is_bool_or_none,
                                               is_flag=True,
                                               )] = False,

    output_file: Annotated[Path | None, OutputFileOption(Img2PDFBackend)] = None,
):
    execute_image_to_pdf_cmd(
        input_files=input_files,
        dpi=dpi,
        fit=fit,
        page_size=page_size,
        set_metadata=set_metadata,
        output_file=output_file,
    )


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
    "execute_image_to_pdf_cmd",
]



# src\file_conversor\backend\pdf2docx_backend.py

"""
This module provides functionalities for handling files using ``pdf2docx`` backend.
"""
import pdf2docx
from pathlib import Path

from typing import Any, Callable

# user-provided imports
from file_conversor.config import Environment, Log, State
from file_conversor.config.locale import get_translation

from file_conversor.backend.abstract_backend import AbstractBackend

STATE = State.get_instance()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)
_ = get_translation()


class PDF2DOCXBackend(AbstractBackend):
    """
    A class that provides an interface for handling files using ``pdf2docx``.
    """

    SUPPORTED_IN_FORMATS = {
        "pdf": {},
    }
    SUPPORTED_OUT_FORMATS = {
        "docx": {},
    }
    EXTERNAL_DEPENDENCIES: set[str] = set()

    def __init__(
        self,
        verbose: bool = False,
    ):
        """
        Initialize the backend

        :param verbose: Verbose logging. Defaults to False.      
        """
        super().__init__()
        self._verbose = verbose

    def convert(self,
                output_file: str | Path,
                input_file: str | Path,
                password: str | None = None,
                ):
        """
        Convert input files into output file.

        :param output_file: Output file
        :param input_file: Input file. 
        :param password: Password for encrypted PDF files. Defaults to "".

        :raises FileNotFoundError: if input file not found
        :raises ValueError: if output format is unsupported
        """

        input_file = Path(input_file).resolve()
        output_file = Path(output_file).resolve()

        self.check_file_exists(input_file)

        converter = pdf2docx.Converter(
            str(input_file),
            password=password,  # pyright: ignore[reportArgumentType]
        )
        if converter.fitz_doc.is_encrypted and not password:
            raise ValueError(_("Password is required for encrypted PDF files"))
        converter.convert(
            str(output_file),
        )


__all__ = [
    "PDF2DOCXBackend",
]



# src\file_conversor\cli\gui\_typer.py

# user-provided modules
from file_conversor.config import get_translation

_ = get_translation()

# command
COMMAND_NAME = "gui"

# SUBCOMMANDS
START_NAME = "start"

__all__ = [
    "COMMAND_NAME",

    "START_NAME",
]



# src/file_conversor/backend/gui/_api/image/to_pdf.py

from flask import json, render_template, request, url_for
from pathlib import Path
from typing import Any

# user-provided modules
from file_conversor.backend.gui.flask_api import FlaskApi
from file_conversor.backend.gui.flask_api_status import FlaskApiStatus

from file_conversor.cli.image.to_pdf_cmd import execute_image_to_pdf_cmd, EXTERNAL_DEPENDENCIES

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


def _api_thread(params: dict[str, Any], status: FlaskApiStatus) -> None:
    """Thread to handle image to PDF conversion."""
    logger.debug(f"Image to PDF thread received: {params}")

    input_files = [Path(i) for i in params['input-files']]
    output_file = Path(params['output-file'])

    image_dpi = int(params['image-dpi'])
    image_fit = str(params['image-fit'])
    image_page_size = params['image-page-size'] or None
    image_set_metadata = bool(params['image-set-metadata'])

    execute_image_to_pdf_cmd(
        input_files=input_files,
        dpi=image_dpi,
        fit=image_fit,
        page_size=image_page_size,
        set_metadata=image_set_metadata,
        output_file=output_file,
        progress_callback=status.set_progress,
    )


def api_image_to_pdf():
    """API endpoint to convert image files to PDF."""
    logger.info(f"[bold]{_('Image to PDF requested via API.')}[/]")
    return FlaskApi.execute_response(_api_thread)


__all__ = [
    "EXTERNAL_DEPENDENCIES",
    "api_image_to_pdf",
]



# src/file_conversor/utils/abstract_singleton_thread_safe.py

import threading
from typing import Self


class AbstractSingletonThreadSafe:
    _instances = {}
    _lock = threading.RLock()

    @classmethod
    def get_instance(cls, *args, **kwargs) -> Self:
        with cls._lock:
            if cls not in cls._instances:
                cls._instances[cls] = cls(*args, **kwargs)
        return cls._instances[cls]


__all__ = [
    "AbstractSingletonThreadSafe",
]




# src\file_conversor\cli\config\show_cmd.py

import typer

from typing import Annotated

from rich import print
from rich.pretty import Pretty

# user-provided modules
from file_conversor.cli.config._typer import COMMAND_NAME, SHOW_NAME
from file_conversor.config import Configuration, State, Log, get_translation

# app configuration
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

# create command
typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = set()


# config show
@typer_cmd.command(
    name=SHOW_NAME,
    help=f"""
        {_('Show the current configuration of the application')}.
    """,
    epilog=f"""
    **{_('Examples')}:** 

        - `file_conversor {COMMAND_NAME} {SHOW_NAME}`
    """
)
def show():
    print(f"{_('Configuration')}:", Pretty(CONFIG.to_dict(), expand_all=True))


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
]




# src\file_conversor\cli\video\execute_cmd.py

import shlex
import typer

from rich import print

from typing import Annotated, List
from pathlib import Path

# user-provided modules
from file_conversor.backend import FFmpegBackend
from file_conversor.backend.audio_video.ffmpeg_filter import FFmpegFilter

from file_conversor.cli.video._typer import OTHERS_PANEL as RICH_HELP_PANEL
from file_conversor.cli.video._typer import COMMAND_NAME, EXECUTE_NAME
from file_conversor.config import Environment, Configuration, State, Log, get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.validators import check_valid_options
from file_conversor.utils.typer_utils import AudioBitrateOption, AudioCodecOption, FormatOption, InputFilesArgument, OutputDirOption, VideoBitrateOption, VideoCodecOption

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = FFmpegBackend.EXTERNAL_DEPENDENCIES


@typer_cmd.command(
    name=EXECUTE_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Execute FFmpeg command (advanced, use with caution).')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor {COMMAND_NAME} {EXECUTE_NAME} input_file.webm -od output_dir/ -f mp4 --audio-bitrate 192`

        - `file_conversor {COMMAND_NAME} {EXECUTE_NAME} input_file.mp4 -f mp3 -fa "-c:a libmp3lame -pass 1"`
    """)
def execute(
    input_files: Annotated[List[Path], InputFilesArgument(FFmpegBackend)],

    file_format: Annotated[str, FormatOption(FFmpegBackend)],

    audio_bitrate: Annotated[int, AudioBitrateOption()] = CONFIG["audio-bitrate"],

    video_bitrate: Annotated[int, VideoBitrateOption()] = CONFIG["video-bitrate"],

    audio_codec: Annotated[str | None, AudioCodecOption(FFmpegBackend.get_supported_audio_codecs())] = None,

    video_codec: Annotated[str | None, VideoCodecOption(FFmpegBackend.get_supported_video_codecs())] = None,

    audio_filters: Annotated[List[str], typer.Option("--audio-filter", "-af",
                                                     help=f'{_("Apply a custom FFmpeg audio filter")} {_("(advanced option, use with caution). Uses the same format as FFmpeg filters (e.g., filter=option1=value1:option2=value2:...). Filters are applied in the order they appear in the command.")}. {_('Defaults to None (do not apply custom filters)')}.',
                                                     )] = [],

    video_filters: Annotated[List[str], typer.Option("--video-filter", "-vf",
                                                     help=f'{_("Apply a custom FFmpeg video filter")} {_("(advanced option, use with caution). Uses the same format as FFmpeg filters (e.g., filter=option1=value1:option2=value2:...). Filters are applied in the order they appear in the command.")}. {_('Defaults to None (do not apply custom filters)')}.',
                                                     )] = [],

    ffmpeg_args: Annotated[str, typer.Option("--ffmpeg-args", "-fa",
                                             help=f'{_("Apply a custom FFmpeg output arguments (advanced option, use with caution).")}. {_('Defaults to None')}.',
                                             )] = "",

    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    # init ffmpeg
    ffmpeg_backend = FFmpegBackend(
        install_deps=CONFIG['install-deps'],
        verbose=STATE["verbose"],
        overwrite_output=STATE["overwrite-output"],
    )

    # set filters
    audio_filters_obj = [FFmpegFilter.from_str(f) for f in audio_filters]
    video_filters_obj = [FFmpegFilter.from_str(f) for f in video_filters]

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        ffmpeg_backend.set_files(input_file=input_file, output_file=output_file)
        ffmpeg_backend.set_audio_codec(codec=audio_codec, bitrate=audio_bitrate, filters=audio_filters_obj)
        ffmpeg_backend.set_video_codec(codec=video_codec, bitrate=video_bitrate, filters=video_filters_obj)

        # display current progress
        ffmpeg_backend.execute(
            progress_callback=progress_mgr.update_progress,
            out_opts=shlex.split(ffmpeg_args),
        )
        progress_mgr.complete_step()

    cmd_mgr = CommandManager(input_files, output_dir=output_dir, overwrite=STATE["overwrite-output"])
    cmd_mgr.run(callback, out_suffix=f".{file_format}")

    logger.info(f"{_('FFMpeg execution')}: [green][bold]{_('SUCCESS')}[/bold][/green]")


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
]



# src\file_conversor\utils\bulma\form.py

from typing import Any

from file_conversor.utils.dominate_utils import button, div


def ButtonsContainer(
    *buttons: Any,
    _class: str = "",
    **kwargs,
):
    """
    Create a container for buttons.

    :param buttons: The button elements to include in the container.
    :param _class: Additional CSS classes for the container.
    """
    return div(
        *buttons,
        _class=f"buttons {_class}",
        **kwargs,
    )


def Button(
    *content: Any,
    _class: str = "",
    _type: str = "button",
    _title: str = "",
    **kwargs,
):
    """
    Create a button element.

    :param content: The content for the button. Can be a label, icon element, or other.
    :param _class: Additional CSS classes for the button.
    :param _type: The type attribute for the button.
    :param _title: The help text for the button (used as title attribute).
    """
    return button(
        *[
            c
            for c in content
            if c is not None
        ],
        _class=f"button {_class}",
        _type=_type,
        _title=_title,
        **kwargs,
    )


__all__ = [
    "ButtonsContainer",
    "Button",
]




# src\file_conversor\cli\audio\__init__.py


import typer

# user-provided modules
from file_conversor.config.locale import get_translation

from file_conversor.cli.audio._typer import COMMAND_NAME

from file_conversor.cli.audio.check_cmd import typer_cmd as check_cmd
from file_conversor.cli.audio.convert_cmd import typer_cmd as convert_cmd
from file_conversor.cli.audio.info_cmd import typer_cmd as info_cmd

_ = get_translation()

audio_cmd = typer.Typer(
    name=COMMAND_NAME,
    help=_("Audio file manipulation (requires FFMpeg external library)"),
)

# CONVERSION_PANEL
audio_cmd.add_typer(convert_cmd)
audio_cmd.add_typer(info_cmd)
audio_cmd.add_typer(check_cmd)

__all__ = [
    "audio_cmd",
]




# src\file_conversor\cli\pdf\repair_cmd.py

import typer

from pathlib import Path
from typing import Callable, Any, Annotated, List

from rich import print

# user-provided modules
from file_conversor.backend.pdf import PikePDFBackend

from file_conversor.cli.pdf._typer import OTHERS_PANEL as RICH_HELP_PANEL
from file_conversor.cli.pdf._typer import COMMAND_NAME, REPAIR_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.typer_utils import InputFilesArgument, OutputDirOption, PasswordOption

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = PikePDFBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    for ext in PikePDFBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="repair",
                description="Repair",
                command=f'cmd.exe /c "{Environment.get_executable()} "{COMMAND_NAME}" "{REPAIR_NAME}" "%1""',
                icon=str(icons_folder_path / 'repair.ico'),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_pdf_repair_cmd(
    input_files: List[Path],
    password: str | None,
    output_dir: Path,
    progress_callback: Callable[[float], Any] = lambda p: p,
):
    pikepdf_backend = PikePDFBackend(verbose=STATE["verbose"])

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        print(f"Processing '{output_file}' ... ")
        pikepdf_backend.compress(
            # files
            input_file=input_file,
            output_file=output_file,

            # options
            decrypt_password=password,
            progress_callback=lambda p: progress_callback(progress_mgr.update_progress(p))
        )
        progress_callback(progress_mgr.complete_step())

    cmd_mgr = CommandManager(input_files, output_dir=output_dir, overwrite=STATE["overwrite-output"])
    cmd_mgr.run(callback, out_stem="_repaired")

    logger.info(f"{_('Repair PDF')}: [bold green]{_('SUCCESS')}[/].")


@typer_cmd.command(
    name=REPAIR_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Attempt to repair a corrupted PDF file.')}        
        
        {_('Outputs a file with _repaired at the end.')}
    """,
    epilog=f"""
**{_('Examples')}:** 

- `file_conversor {COMMAND_NAME} {REPAIR_NAME} input_file.pdf -od D:/Downloads` 
""")
def repair(
    input_files: Annotated[List[Path], InputFilesArgument(PikePDFBackend)],
    password: Annotated[str | None, PasswordOption()] = None,
    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    execute_pdf_repair_cmd(
        input_files=input_files,
        password=password,
        output_dir=output_dir,
    )


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
    "execute_pdf_repair_cmd",
]



# src/file_conversor/backend/gui/_api/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui._api.config import api_config
from file_conversor.backend.gui._api.status import api_status

# sub APIs
from file_conversor.backend.gui._api.audio import routes as audio_api_routes
from file_conversor.backend.gui._api.doc import routes as doc_api_routes
from file_conversor.backend.gui._api.ebook import routes as ebook_api_routes
from file_conversor.backend.gui._api.hash import routes as hash_api_routes
from file_conversor.backend.gui._api.image import routes as image_api_routes
from file_conversor.backend.gui._api.pdf import routes as pdf_api_routes
from file_conversor.backend.gui._api.ppt import routes as ppt_api_routes
from file_conversor.backend.gui._api.text import routes as text_api_routes
from file_conversor.backend.gui._api.video import routes as video_api_routes
from file_conversor.backend.gui._api.xls import routes as xls_api_routes


def routes():
    return [
        # general APIs
        FlaskRoute(
            rule="/api/config",
            handler=api_config,
            methods=["GET", "POST"]
        ),
        FlaskRoute(
            rule="/api/status",
            handler=api_status,
        ),
        # plugin APIs
        *audio_api_routes(),
        *doc_api_routes(),
        *ebook_api_routes(),
        *hash_api_routes(),
        *image_api_routes(),
        *pdf_api_routes(),
        *ppt_api_routes(),
        *text_api_routes(),
        *video_api_routes(),
        *xls_api_routes(),
    ]


__all__ = ['routes']



# src\file_conversor\backend\gui\audio\_dom_page.py

from flask import url_for
from typing import Any

# user-provided modules
from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def audio_index_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Audio"),
        'url': url_for('audio_index'),
        'active': active,
    }


def audio_check_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Check"),
        'url': url_for('audio_check'),
        'active': active,
    }


def audio_convert_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Convert"),
        'url': url_for('audio_convert'),
        'active': active,
    }


def audio_info_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Info"),
        'url': url_for('audio_info'),
        'active': active,
    }


__all__ = [
    "audio_index_nav_item",
    "audio_check_nav_item",
    "audio_convert_nav_item",
    "audio_info_nav_item",
]




# src\file_conversor\cli\image\render_cmd.py

import typer

from pathlib import Path
from typing import Annotated, Any, Callable, List
from rich import print

# user-provided modules
from file_conversor.backend.image import PyMuSVGBackend

from file_conversor.cli.image._typer import CONVERSION_PANEL as RICH_HELP_PANEL
from file_conversor.cli.image._typer import COMMAND_NAME, RENDER_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.typer_utils import DPIOption, FormatOption, InputFilesArgument, OutputDirOption

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = PyMuSVGBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    # PyMuSVGBackend commands
    for ext in PyMuSVGBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="to_jpg",
                description="To JPG",
                command=f'cmd.exe /c "{Environment.get_executable()} "{COMMAND_NAME}" "{RENDER_NAME}" "%1" -f "jpg""',
                icon=str(icons_folder_path / 'jpg.ico'),
            ),
            WinContextCommand(
                name="to_png",
                description="To PNG",
                command=f'cmd.exe /c "{Environment.get_executable()} "{COMMAND_NAME}" "{RENDER_NAME}" "%1" -f "png""',
                icon=str(icons_folder_path / 'png.ico'),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_image_render_cmd(
    input_files: List[Path],
    format: str,
    dpi: int,
    output_dir: Path,
    progress_callback: Callable[[float], Any] = lambda p: p,
):
    pymusvg_backend = PyMuSVGBackend(verbose=STATE['verbose'])

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        logger.info(f"Processing '{output_file}' ... ")
        pymusvg_backend.convert(
            input_file=input_file,
            output_file=output_file,
            dpi=dpi,
        )
        progress_callback(progress_mgr.complete_step())

    cmd_mgr = CommandManager(input_files, output_dir=output_dir, overwrite=STATE["overwrite-output"])
    cmd_mgr.run(callback, out_suffix=f".{format}")

    logger.info(f"{_('Image render')}: [green bold]{_('SUCCESS')}[/]")


@typer_cmd.command(
    name=RENDER_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Render an image vector file into a different format.')}
    """,
    epilog=f"""
        **{_('Examples')}:**

        - `file_conversor {COMMAND_NAME} {RENDER_NAME} input_file.svg -f png`

        - `file_conversor {COMMAND_NAME} {RENDER_NAME} input_file.svg input_file2.svg -od D:/Downloads -f jpg --dpi 300`
    """)
def render(
    input_files: Annotated[List[Path], InputFilesArgument(PyMuSVGBackend)],
    format: Annotated[str, FormatOption(PyMuSVGBackend)],
    dpi: Annotated[int, DPIOption()] = CONFIG["image-dpi"],
    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    execute_image_render_cmd(
        input_files=input_files,
        format=format,
        dpi=dpi,
        output_dir=output_dir,
    )


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
    "execute_image_render_cmd",
]



# src\file_conversor\system\dummy\utils.py

import os


def is_admin() -> bool:
    """True if app running with admin priviledges, False otherwise."""
    return False


def reload_user_path():
    """Reload user PATH in current process."""
    # dummy method
    pass


def set_window_icon(
    window_title: str,
    icon_path: str | os.PathLike,
    cx: int = 0,
    cy: int = 0,
):
    """
    Set the icon of a window given its title.
    Note: Setting window icons is not supported in this dummy implementation.
    """
    # dummy method
    pass


__all__ = [
    "is_admin",
    "reload_user_path",
    "set_window_icon",
]



# src\file_conversor\utils\bulma\form.py

from typing import Any

from file_conversor.utils.dominate_bulma.font_awesome_icon import FontAwesomeIcon
from file_conversor.utils.dominate_utils import div, label, p

from file_conversor.utils.formatters import format_py_to_js


def _validate_func_js(validation_expr: str, value: str, is_valid_var: str) -> str:
    return f"""
        console.log('Validating field with value:', {value});
        {is_valid_var} = {validation_expr} ;
        const parentForm = this.$el.closest('form[x-data]');
        if(parentForm){{
            const parentData = Alpine.$data(parentForm);
            parentData.updateValidity();
        }} else {{
            console.log('No parent form found');
        }}
        return {is_valid_var} ;
    """


def _grouped_field(*input_el: Any, _class_control: str) -> list[Any]:
    controls = []
    for el in input_el:
        if not el:
            continue
        with div(_class=f"control {_class_control}") as control_group:
            control_group.add(el)
        controls.append(control_group)
    return controls


def _addons_field(*input_el: Any) -> list[Any]:
    with div(_class=f"field has-addons") as control_group:
        for el in input_el:
            if not el:
                continue
            control_group.add(el)
    return [control_group]


def _icon_component(icons: dict[str, Any], position: str) -> list[Any]:
    icon_fields = []
    if icons and icons.get(position):
        icon_fields.append(FontAwesomeIcon(icons[position], _class=f"is-{position} is-small"))
    return icon_fields


def _icons_field(*input_el: Any, icons: dict[str, Any], _class_control: str) -> list[Any]:
    _icons_class = ''
    _icons_class += 'has-icons-left' if icons.get('left') else ''
    _icons_class += 'has-icons-right' if icons.get('right') else ''
    with div(_class=f"control {_class_control}") as control:
        for el in input_el:
            if not el:
                continue
            control.add(el)
        control.add(*_icon_component(icons, 'left'))
        control.add(*_icon_component(icons, 'right'))
    return [control]


def _label_component(label_text: str) -> list[Any]:
    components = []
    if label_text:
        components.append(
            label(label_text, _class="label")
        )
    return components


def _help_component(help_var: str, is_hidden: str) -> list[Any]:
    return [
        p(
            _class=f"help is-danger",
            **{
                'x-text': f'{help_var}',
                ':class': f"""{{
                    'is-hidden': {is_hidden},
                }}""",
            },
        )  # Placeholder for error messages
    ]


def FormField(
    *input_el,
    label_text: str = "",
    help: str = "",
    icons: dict[str, Any] | None = None,
    current_value: Any = None,
    validation_expr: str = "true",
    has_addons: bool = False,
    _class: str = "",
    _class_control: str = "",
    x_data: str = "",
    x_init: str = "",
    **kwargs,
):
    """
    Create a form field with a label and optional icons.

    :param input_el: The input elements (e.g., input, textarea, select).
    :param label_text: The text for the label.
    :param help: Optional help text.
    :param icons: Optional icons for left and right (e.g., {"left": left_icon_name, "right": right_icon_name}).
    :param _class: Additional CSS classes for the form field.
    :param _class_control: Additional CSS classes for the control div.
    :param x_data: Additional Alpine.js x-data properties.
    :param x_init: Additional Alpine.js x-init code.
    """
    with div(
        _class=f"field is-full-width {_class}",
        **{
            'x-data': f"""{{
                help: {format_py_to_js(help)},
                value: {format_py_to_js(current_value)},
                isValid: false,
                validate(value){{
                    {_validate_func_js(validation_expr, value="value", is_valid_var="this.isValid")}
                }},
                init() {{
                    this.$watch('value', this.validate.bind(this));
                    this.validate(this.value);   
                    {x_init} ;                 
                }},
                {x_data}
            }}""",
        },
        **kwargs,
    ) as field:
        field.add(*_label_component(label_text))
        if "is-grouped" in _class:
            field.add(*_grouped_field(*input_el, _class_control=_class_control))
        elif has_addons:
            field.add(*_addons_field(*input_el))
        else:
            field.add(*_icons_field(*input_el, icons=icons or {}, _class_control=_class_control))
        field.add(*_help_component("help", is_hidden="isValid"))
    return field


def FormFieldHorizontal(
    *form_fields,
    label_text: str,
    _class: str = "",
    _class_label: str = "is-normal",
    _class_body: str = "",
    **kwargs,
):
    """
    Create a horizontal form field with a label and multiple input elements.

    :param form_fields: The form field elements (e.g., FormField).
    :param label_text: The text for the label.
    :param _class: Additional CSS classes for the form field.
    :param _class_label: Additional CSS classes for the label container.
    :param _class_body: Additional CSS classes for the body container.
    :param kwargs: Additional attributes for the field container.
    """
    with div(_class=f"field is-horizontal is-full-width {_class}", **kwargs) as field:
        with div(_class=f"field-label {_class_label}") as field_label:
            field_label.add(*_label_component(label_text))
        with div(_class=f"field-body {_class_body}") as field_body:
            field_body.add(*form_fields)
    return field


__all__ = [
    "FormFieldHorizontal",
    "FormField",
]



# src/file_conversor/backend/gui/_api/audio/info.py

from flask import json, render_template, request, url_for
from pathlib import Path
from typing import Any

# user-provided modules
from file_conversor.backend.gui.flask_api import FlaskApi, FlaskApiStatus
from file_conversor.backend.gui._api.video.info import _api_thread as _api_thread_info_video, EXTERNAL_DEPENDENCIES

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def _api_thread(params: dict[str, Any], status: FlaskApiStatus) -> None:
    """Thread to handle audio information retrieval."""
    logger.debug(f"Audio info thread received: {params}")
    _api_thread_info_video(params, status)


def api_audio_info():
    """API endpoint to get audio file information."""
    logger.info(f"[bold]{_('Audio info requested via API.')}[/]")
    return FlaskApi.execute_response(_api_thread)


__all__ = [
    "api_audio_info",
    "EXTERNAL_DEPENDENCIES",
]



# src\file_conversor\utils\flask_utils.py

from typing import Callable

from flask import url_for


def get_url(func_or_endpoint: Callable | str) -> str:
    """
    Gets the URL for a Flask endpoint function.

    :param func_or_endpoint: The Flask endpoint function or endpoint name.
    :return: The URL for the given endpoint.
    """
    if isinstance(func_or_endpoint, str):
        return url_for(func_or_endpoint)
    return url_for(func_or_endpoint.__name__)


__all__ = [
    "get_url",
]




# src\file_conversor\cli\audio\info_cmd.py

import typer

from rich import print

from typing import Annotated, List
from pathlib import Path

# user-provided modules
from file_conversor.backend import FFprobeBackend
from file_conversor.cli.audio._typer import COMMAND_NAME, INFO_NAME

from file_conversor.cli.video.info_cmd import info as info_video_cmd, EXTERNAL_DEPENDENCIES

from file_conversor.config import Environment, Configuration, State, Log, get_translation

from file_conversor.utils.typer_utils import InputFilesArgument

from file_conversor.system.win import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()


def register_ctx_menu(ctx_menu: WinContextMenu):
    # FFMPEG commands
    icons_folder_path = Environment.get_icons_folder()
    for ext in FFprobeBackend.SUPPORTED_IN_AUDIO_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="info",
                description="Get Info",
                command=f'cmd.exe /k "{Environment.get_executable()} "{COMMAND_NAME}" "{INFO_NAME}" "%1""',
                icon=str(icons_folder_path / 'info.ico'),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


@typer_cmd.command(
    name=INFO_NAME,
    help=f"""
        {_('Get information about a audio file.')}

        {_('This command retrieves metadata and other information about the audio file')}:

        - {_('Format')} (mp3, m4a, etc)

        - {_('Duration')} (HH:MM:SS)

        - {_('Other properties')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor {COMMAND_NAME} {INFO_NAME} filename.m4a`

        - `file_conversor {COMMAND_NAME} {INFO_NAME} other_filename.mp3`
    """)
def info(
    input_files: Annotated[List[Path], InputFilesArgument(FFprobeBackend.SUPPORTED_IN_AUDIO_FORMATS)],
):
    info_video_cmd(input_files)


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
]



# src\file_conversor\utils\formatters.py

import json
import math
import re
import traceback
import typer

from pathlib import Path
from typing import Any, Iterable, Sequence

# user-provided modules
from file_conversor.config import State, get_translation

from file_conversor.utils.command_manager import CommandManager

STATE = State.get_instance()
_ = get_translation()


def escape_xml(text: Any | str | None) -> str:
    """
    Escape invalid characters for XML.
    """
    if text is None:
        return ""
    text = str(text)
    return (
        text.replace("&", "&amp;")
            .replace("<", "&lt;")
            .replace(">", "&gt;")
            .replace('"', "&quot;")
            .replace("'", "&apos;")
    )


def parse_traceback_list(exc: Exception) -> list[str]:
    """
    Parse traceback to get specific frame.

    :param exc: Exception object.

    :return: List of str.
    """
    return traceback.format_exc().splitlines()


def parse_js_to_py(value: str) -> Any:
    # Attempt to convert to appropriate type
    # Handle null value
    if value is None or value.lower() in ('', 'null', 'none', 'undefined'):
        return None
    # Handle boolean values
    elif value.lower() in ('true', 'on'):
        return True
    elif value.lower() in ('false', 'off'):
        return False
    # Handle numeric values
    elif re.match(r'^-?\d+([\.,]\d+)?$', value):
        if '.' in value or ',' in value:
            return float(value.replace(',', '.'))
        else:
            return int(value)
    # Handle JSON arrays and objects
    elif value[0] in ('[', '{',) and value[-1] in (']', '}',):
        return json.loads(value.replace("'", '"'))
    return value  # return as string


def parse_ffmpeg_filter(filter: str | None) -> tuple[str, list, dict]:
    """
    Parse FFmpeg filter string to name, args, kwargs.

    :param filter: FFmpeg filter string.
    :returns: (name, args, kwargs)
    """
    if not filter:
        return "", [], {}
    splitted = filter.split("=", maxsplit=1)
    name = splitted[0]
    args = splitted[1].split(":") if len(splitted) > 1 else []
    kwargs = {}
    for arg in args.copy():
        arg_splitted = arg.split("=")
        arg_name = arg_splitted[0]
        arg_data = arg_splitted[1] if len(arg_splitted) > 1 else ""
        if not arg_data:
            continue
        kwargs[arg_name] = arg_data
        args.remove(arg)
    return name, args, kwargs


def parse_image_resize_scale(scale: float | None, width: int | None, quiet: bool):
    if not scale and not width:
        if quiet:
            raise RuntimeError(f"{_('Scale and width not provided')}")
        userinput = str(typer.prompt(f"{_('Output image scale (e.g., 1.5)')}"))
        scale = float(userinput)
    return scale


def parse_pdf_rotation(rotation: list[str], last_page: int) -> dict[int, int]:
    """Parse PDF rotation argument to dict of page: degree (0-based)."""
    # get rotation dict in format {page: rotation}
    rotation_dict = {}
    for arg in rotation:
        match = re.search(r'(\d+)(-(\d*))?:(-?\d+)', arg)
        if not match:
            raise RuntimeError(f"{_('Invalid rotation instruction')} '{arg}'. {_("Valid format is 'begin-end:degree' or 'page:degree'")}.")

        # check user input
        begin = int(match.group(1)) - 1
        end = begin
        if match.group(3):
            end = int(match.group(3)) - 1
        elif match.group(2):
            end = last_page - 1
        degree = int(match.group(4))
        if end < begin:
            raise RuntimeError(f"{_('Invalid begin-end page interval')}. {_('End Page < Begin Page')} '{arg}'.")

        # create rotation_dict
        for page_num in range(begin, end + 1):
            rotation_dict[page_num] = degree
    return rotation_dict


def parse_pdf_pages(pages: list[str] | str | None) -> list[int]:
    """Parse PDF pages argument to list of page numbers (0-based)."""
    if not pages:
        pages_str = typer.prompt(f"{_('Pages to extract [comma-separated list] (e.g., 1-3, 7-7)')}")
        pages = str(pages_str)

    if isinstance(pages, str):
        pages = [p.strip() for p in pages.split(",")]

    # parse user input
    pages_list: list[int] = []
    for arg in pages:
        match = re.compile(r'(\d+)(-(\d*))?').search(arg)
        if not match:
            raise RuntimeError(f"{_('Invalid page instruction')} '{arg}'. {_("Valid format is 'page' or 'begin-end'")}.")

        # check user input
        begin = int(match.group(1)) - 1
        end = int(match.group(3)) - 1 if match.group(3) else begin

        if end < begin:
            raise RuntimeError(f"{_('Invalid begin-end page interval')}. {_('End Page < Begin Page')} '{arg}'.")

        # create pages list
        pages_list.extend(range(begin, end + 1))
    return pages_list


def normalize_degree(deg: float | int) -> int:
    """Normalize clockwise degree to 0-360"""
    # parse rotation argument
    degree = int(math.fmod(deg, 360))
    if degree < 0:
        degree += 360  # fix rotation signal
    return degree


def parse_bytes(target_size: str | None) -> int:
    """
    Parse file size string (e.g., 100.5M, 2G) to bytes. 

    :return: Size in bytes.
    """
    if not target_size or target_size == "0":
        return 0
    size_unit = target_size[-1].upper()
    size_value = float(target_size[:-1])
    if size_unit.isdigit():
        return round(float(target_size))
    elif size_unit == "K":
        return round(size_value * 1024.0)
    elif size_unit == "M":
        return round(size_value * 1024.0 * 1024.0)
    elif size_unit == "G":
        return round(size_value * 1024.0 * 1024.0 * 1024.0)
    elif size_unit == "T":
        return round(size_value * 1024.0 * 1024.0 * 1024.0 * 1024.0)
    raise ValueError(f"{_('Invalid size format')} '{target_size}'.")


def format_bytes(size: float | int) -> str:
    """Format size in bytes, KB, MB, GB, or TB"""
    # Size in bytes to a human-readable string
    size = float(size)
    for unit in ['B', 'KB', 'MB', 'GB', 'TB']:
        if size < 1024.0:
            return f"{size:.1f} {unit}"
        size /= 1024.0
    return f"{size:.1f} TB"


def format_bitrate(bps: int) -> str:
    """Format bitrate in bps, kbps, Mbps, Gbps, Tbps etc."""
    if bps >= 1024 * 1024 * 1024 * 1024:
        return f"{bps / (1024 * 1024 * 1024 * 1024):.1f} Tbps"
    if bps >= 1024 * 1024 * 1024:
        return f"{bps / (1024 * 1024 * 1024):.1f} Gbps"
    if bps >= 1024 * 1024:
        return f"{bps / (1024 * 1024):.1f} Mbps"
    elif bps >= 1024:
        return f"{bps / 1024:.1f} Kbps"
    return f"{bps} bps"


def format_alphanumeric(text: str) -> str:
    """Format text to be alphanumeric only (remove non-alphanumeric characters)."""
    formatted = text
    formatted = re.sub(r'[áàâäãå]', 'a', formatted)
    formatted = re.sub(r'[éèëê]', 'e', formatted)
    formatted = re.sub(r'[íïìî]', 'i', formatted)
    formatted = re.sub(r'[óòöôõ]', 'o', formatted)
    formatted = re.sub(r'[úüùû]', 'u', formatted)
    formatted = re.sub(r'[çc]', 'c', formatted)
    formatted = re.sub(r'[ñn]', 'n', formatted)
    formatted = re.sub(r'[\s]+', ' ', formatted)
    formatted = re.sub(r'[^a-zA-Z0-9_ ]', '', formatted)
    return formatted.strip()


def format_file_types_webview(*file_types: str, description: str = "") -> str:
    """
    Format file types for PyWebView file dialogs.

    :param file_types: File type patterns (e.g., '*.png', '*.jpg').
    :param description: Description for the file types (e.g., 'Image Files').

    :return: Formatted file types string for PyWebView.
    """
    if not file_types:
        return f'{format_alphanumeric(_("All Files"))} (*.*)'
    if not description:
        description = _("Custom Files")
    parsed_types = [ft if ft.startswith("*.") else f"*.{ft.lstrip('.')}" for ft in file_types]
    return f'{format_alphanumeric(description)} ({";".join(parsed_types)})'


def format_py_to_js(value: Any) -> str:
    """Convert Python value to JavaScript-compatible string."""
    if isinstance(value, Path):
        value = str(value.resolve())
    data = json.dumps(value)
    if data[0] in ('"', "'") and data[-1] in ('"', "'"):
        # Strip quotes and wrap in backticks
        return f"`{data[1:-1]}`"
    return data


def format_traceback_str(exc: Exception, debug: bool = True) -> str:
    """
    Format exception traceback as string.

    :param exc: Exception object.
    :param debug: bool: Whether to include detailed traceback information.

    :return: Formatted traceback string.
    """
    exc_formatted = f'[bold red]{type(exc).__name__}[/]: {str(exc)}'
    if not debug:
        return exc_formatted

    stack_str_list = parse_traceback_list(exc)
    return '\n'.join(stack_str_list[:-1] + [exc_formatted])


def format_traceback_html(exc: Exception, debug: bool = True) -> str:
    """
    Format exception traceback as HTML.

    :param exc: Exception object.
    :param debug: bool: Whether to include detailed traceback information.

    :return: Formatted traceback HTML.
    """
    tab = "&nbsp;&nbsp;"
    exc_formatted = f'<b style="color:red;">{type(exc).__name__}</b>: {str(exc)}'
    if not debug:
        return exc_formatted

    stack_str_list = parse_traceback_list(exc)
    return '<br>'.join([
        escape_xml(s).replace(' ', "&nbsp;").replace('\t', tab).replace('\n', '<br>')
        for s in stack_str_list[:-1]
    ] + [exc_formatted])


def format_in_out_files_tuple(
    input_files: list[Path],
    format: str,
    output_dir: Path,
):
    """
    Get input and output files for conversion.

    :param input_files: List of input file paths.
    :param format: Output file format.
    :param output_dir: Output directory path.

    :raises FileExistsError: if output file exists and overwrite is disabled.
    """
    files = [
        (input_file, output_dir / CommandManager.get_output_file(input_file, suffix=f".{format.lstrip('.')}"))
        for input_file in input_files
    ]
    for input_file, output_file in files:
        if not STATE["overwrite-output"] and output_file.exists():
            raise FileExistsError(f"{_("File")} '{output_file}' {_("exists")}. {_("Use")} 'file_conversor -oo' {_("to overwrite")}.")
    return files


__all__ = [
    "escape_xml",
    "parse_traceback_list",
    "parse_js_to_py",
    "parse_ffmpeg_filter",
    "parse_image_resize_scale",
    "parse_pdf_rotation",
    "parse_pdf_pages",
    "normalize_degree",
    "parse_bytes",
    "format_bytes",
    "format_bitrate",
    "format_alphanumeric",
    "format_file_types_webview",
    "format_py_to_js",
    "format_traceback_str",
    "format_traceback_html",
    "format_in_out_files_tuple",
]




# src\file_conversor\cli\pdf\split_cmd.py

import typer

from pathlib import Path
from typing import Annotated, Any, Callable, List

from rich import print

# user-provided modules
from file_conversor.backend.pdf import PyPDFBackend

from file_conversor.cli.pdf._typer import TRANSFORMATION_PANEL as RICH_HELP_PANEL
from file_conversor.cli.pdf._typer import COMMAND_NAME, SPLIT_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.typer_utils import InputFilesArgument, OutputDirOption, PasswordOption

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = PyPDFBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    for ext in PyPDFBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="split",
                description="Split",
                command=f'cmd.exe /c "{Environment.get_executable()} "{COMMAND_NAME}" "{SPLIT_NAME}" "%1""',
                icon=str(icons_folder_path / 'split.ico'),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_pdf_split_cmd(
    input_files: List[Path],
    password: str | None,
    output_dir: Path,
    progress_callback: Callable[[float], Any] = lambda p: p,
):
    pypdf_backend = PyPDFBackend(verbose=STATE["verbose"])

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        print(f"Processing '{output_file}' ... ")
        pypdf_backend.split(
            input_file=input_file,
            output_file=output_file,
            password=password,
            progress_callback=lambda p: progress_callback(progress_mgr.update_progress(p)),
        )
        progress_callback(progress_mgr.complete_step())

    cmd_mgr = CommandManager(input_files, output_dir=output_dir, overwrite=True)  # avoid issues with existing files
    cmd_mgr.run(callback)
    logger.info(f"{_('Split pages')}: [bold green]{_('SUCCESS')}[/].")


# pdf split
@typer_cmd.command(
    name=SPLIT_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Split PDF pages into several 1-page PDFs.')}

        {_('For every PDF page, a new single page PDF will be created using the format `input_file_X.pdf`, where X is the page number.')}
    """,
    epilog=f"""
**{_('Examples')}:** 



*{_('Split pages of input_file.pdf into output_file_X.pdf files')}*:

- `file_conversor {COMMAND_NAME} {SPLIT_NAME} input_file.pdf -od D:/Downloads` 



*{_('For every PDF page, generate a "input_file_X.pdf" file')}*:

- `file_conversor {COMMAND_NAME} {SPLIT_NAME} input_file.pdf` 
""")
def split(
    input_files: Annotated[List[Path], InputFilesArgument(PyPDFBackend)],
    password: Annotated[str | None, PasswordOption()] = None,
    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    execute_pdf_split_cmd(
        input_files=input_files,
        password=password,
        output_dir=output_dir,
    )


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
    "execute_pdf_split_cmd",
]



# src/file_conversor/utils/dominate_bulma/progress.py

from file_conversor.utils.dominate_utils import progress


def Progress(
        _max: int | str = 100,
        _class: str = "",
        **kwargs,
):
    """
    Create a progress bar element.

    :param _max: The maximum value of the progress.
    :param _class: Additional CSS classes for the progress bar.
    """
    return progress(_class=f"progress {_class}", _max=str(_max), **kwargs)


__all__ = [
    'Progress',
]




# src\file_conversor\cli\video\_ffmpeg_cmd_helper.py

from rich import print

from typing import Annotated, Any, Callable, List, Self
from pathlib import Path

# user-provided modules
from file_conversor.backend import FFmpegBackend, FFprobeBackend
from file_conversor.backend.audio_video.ffmpeg_filter import FFmpegFilter, FFmpegFilterDeshake, FFmpegFilterEq, FFmpegFilterHflip, FFmpegFilterMInterpolate, FFmpegFilterScale, FFmpegFilterTranspose, FFmpegFilterUnsharp, FFmpegFilterVflip

from file_conversor.config import Log, get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.formatters import format_bytes, normalize_degree, parse_bytes
from file_conversor.utils.validators import check_valid_options, is_close

# get app config
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


EXTERNAL_DEPENDENCIES = FFmpegBackend.EXTERNAL_DEPENDENCIES


class FFmpegCmdHelper:
    def __init__(
            self,
            install_deps: bool | None,
            verbose: bool,
            overwrite_output: bool,
    ) -> None:
        super().__init__()

        # init ffmpeg and ffprobe backends
        self._ffmpeg_backend = FFmpegBackend(
            install_deps=install_deps,
            verbose=verbose,
            overwrite_output=overwrite_output,
        )
        self._ffprobe_backend = FFprobeBackend(
            install_deps=install_deps,
            verbose=verbose,
        )
        self._install_deps = install_deps
        self._verbose = verbose
        self._overwrite_output = overwrite_output

        self._input_files: list[Path] = []

        self._file_format: str = ""
        self._out_stem: str = ""
        self._output_dir: Path = Path()

        self._audio_codec: str | None = None
        self._video_codec: str | None = None

        self._video_encoding_speed: str | None = None
        self._video_quality: str | None = None

        self._target_size_bytes: int = 0

        self._audio_bitrate: int = 0
        self._video_bitrate: int = 0
        self._two_pass: bool = False

        self._audio_filters: list[FFmpegFilter] = []
        self._video_filters: list[FFmpegFilter] = []

    def _set_video_bitrate_for_target_size(self, input_file: Path):
        if self._target_size_bytes <= 0:
            return

        duration = self._ffprobe_backend.get_duration(input_file)
        if duration < 0:
            raise RuntimeError(_('Could not determine input file duration'))

        # total size in kbit
        target_size_kbit = int(self._target_size_bytes * 8.0 / 1024.0)
        target_size_kbps = int(target_size_kbit / duration)

        # audio size
        self._audio_bitrate = 128 if self._audio_bitrate <= 0 else self._audio_bitrate
        audio_kbps = self._audio_bitrate / 8.0
        audio_mb = audio_kbps * duration / 1024.0

        self._video_bitrate = target_size_kbps - self._audio_bitrate
        if self._video_bitrate < 1:
            target_size = format_bytes(self._target_size_bytes)
            raise RuntimeError(f"{_('Target size too small')}: {target_size}. {_(f'Increase target size to at least')} '{audio_mb + 0.100:.2f}M' {_('(might not be enougth to achieve good video quality)')}.")

    def set_input(self, input_files: list[Path]) -> Self:
        """
        Set input files.

        :param input_files: List of input file paths.
        """
        self._input_files = input_files
        return self

    def set_output(
        self,
        file_format: str,
        out_stem: str = "",
        output_dir: Path = Path(),
    ) -> Self:
        """
        Set output parameters.

        :param file_format: Output file format.
        :param out_stem: Output file name stem.
        :param output_dir: Output directory path.
        """
        self._file_format = file_format
        self._out_stem = out_stem
        self._output_dir = output_dir
        return self

    def set_codecs(self, audio_codec: str | None, video_codec: str | None) -> Self:
        """
        Set audio and video codecs.

        :param audio_codec: Audio codec name.
        :param video_codec: Video codec name.
        """
        self._audio_codec = audio_codec
        self._video_codec = video_codec
        return self

    def set_video_settings(self, encoding_speed: str | None = None, quality: str | None = None) -> Self:
        """
        Set video encoding settings.

        :param encoding_speed: Video encoding speed preset.
        :param quality: Video quality setting.
        """
        self._video_encoding_speed = encoding_speed
        self._video_quality = quality
        return self

    def set_bitrate(self, audio_bitrate: int = 0, video_bitrate: int = 0) -> Self:
        """
        Set audio and video bitrate.

        :param audio_bitrate: Audio bitrate value.
        :param video_bitrate: Video bitrate value.
        """
        self._audio_bitrate = audio_bitrate
        self._video_bitrate = video_bitrate
        self._two_pass = (self._video_bitrate > 0) or (self._audio_bitrate > 0)
        return self

    def set_target_size(self, size_str: str | None) -> Self:
        """
        Convert target size string to bytes.

        :param size_str: Target size string (e.g., "100M", "1G").
        :return: Target size in bytes.
        """
        self._target_size_bytes = parse_bytes(size_str) if size_str else 0
        return self

    def set_resolution_filter(self, resolution: str | None) -> Self:
        """
        Set FFmpeg resolution from resolution string.

        :param resolution: Resolution string (e.g., "1920:1080").
        """
        if resolution is None:
            return self
        width_height = resolution.split(":")
        if len(width_height) != 2:
            raise ValueError(f"{_('Invalid resolution format')}: {resolution} {_('(expected format WIDTH:HEIGHT)')}")
        width, height = width_height
        self._video_filters.append(FFmpegFilterScale(width, height))
        return self

    def set_fps_filter(self, fps: int | None) -> Self:
        """
        Set FFmpeg target fps from fps integer value.

        :param fps: Frames per second value.
        """
        if fps is None:
            return self
        if fps < 1:
            raise ValueError(f"{_('Invalid FPS value')}: {fps}")
        self._video_filters.append(FFmpegFilterMInterpolate(fps=fps))
        return self

    def set_enhancement_filters(
        self,
        brightness: float = 1.0,
        contrast: float = 1.0,
        color: float = 1.0,
        gamma: float = 1.0,
    ) -> Self:
        """
        Set FFmpeg enhancement filters.

        :param brightness: Brightness adjustment value.
        :param contrast: Contrast adjustment value.
        :param color: Color adjustment value.
        :param gamma: Gamma adjustment value.
        """
        if not (is_close(brightness, 1.0) and is_close(contrast, 1.0) and is_close(color, 1.0) and is_close(gamma, 1.0)):
            self._video_filters.append(FFmpegFilterEq(brightness=brightness, contrast=contrast, saturation=color, gamma=gamma))
        return self

    def set_rotation_filter(self, rotation: int | None) -> Self:
        """
        Set FFmpeg rotation filter from rotation integer value.

        :param rotation: Rotation degree value.
        """
        if rotation is None:
            return self
        rotation = normalize_degree(rotation)
        if rotation == 0:
            # no need to rotate
            return self
        elif rotation in (90, 270):
            direction = {
                90: 1,
                270: 2,
            }[rotation]
            self._video_filters.append(FFmpegFilterTranspose(direction=direction))
        elif rotation in (180,):
            self._video_filters.append(FFmpegFilterTranspose(direction=1))
            self._video_filters.append(FFmpegFilterTranspose(direction=1))
        else:
            raise ValueError(f"{_('Invalid rotation value')}: {rotation}")
        return self

    def set_mirror_filter(self, mirror_axis: str | None) -> Self:
        """
        Set FFmpeg mirror filter from mirror axis string.

        :param mirror_axis: Mirror axis string ("x", "y", "xy").
        """
        if mirror_axis is None or mirror_axis == "":
            # no need to mirror
            return self
        elif mirror_axis == "x":
            self._video_filters.append(FFmpegFilterHflip())
        elif mirror_axis == "y":
            self._video_filters.append(FFmpegFilterVflip())
        elif mirror_axis == "xy":
            self._video_filters.append(FFmpegFilterHflip())
            self._video_filters.append(FFmpegFilterVflip())
        else:
            raise ValueError(f"{_('Invalid mirror axis value')}: {mirror_axis}")
        return self

    def set_deshake_filter(self, deshake: bool) -> Self:
        """
        Set FFmpeg deshake filter.

        :param deshake: Whether to apply deshake filter.
        """
        if deshake:
            self._video_filters.append(FFmpegFilterDeshake())
        return self

    def set_unsharp_filter(self, unsharp: bool) -> Self:
        """
        Set FFmpeg unsharp filter.

        :param unsharp: Whether to apply unsharp filter.
        """
        if unsharp:
            self._video_filters.append(FFmpegFilterUnsharp())
        return self

    def execute(
        self,
        progress_callback: Callable[[float, ProgressManager], Any] | None = None,
    ) -> Self:
        def _callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
            logger.debug(f"Input file: {input_file}")

            logger.debug(f"{_('Audio bitrate')}: [green][bold]{self._audio_bitrate} kbps[/bold][/green]")
            logger.debug(f"{_('Video bitrate')}: [green][bold]{self._video_bitrate} kbps[/bold][/green]")

            self._ffmpeg_backend.set_files(input_file=input_file, output_file=output_file)
            self._ffmpeg_backend.set_audio_codec(
                codec=self._audio_codec,
                bitrate=self._audio_bitrate,
                filters=self._audio_filters,
            )
            self._ffmpeg_backend.set_video_codec(
                codec=self._video_codec,
                bitrate=self._video_bitrate,
                filters=self._video_filters,
                encoding_speed=self._video_encoding_speed,
                quality_setting=self._video_quality,
            )

            progress_update_cb = progress_mgr.update_progress
            if progress_callback is not None:
                def progress_update_cb(step_progress: float): return progress_callback(step_progress, progress_mgr)  # pyright: ignore[reportOptionalCall]

            progress_complete_cb = progress_mgr.complete_step
            if progress_callback is not None:
                def progress_complete_cb(): return progress_callback(progress_mgr.complete_step(), progress_mgr)  # pyright: ignore[reportOptionalCall]

            # display current progress
            self._ffmpeg_backend.execute(
                progress_callback=progress_update_cb,
                pass_num=1 if self._two_pass else 0,
            )
            progress_complete_cb()

            if self._two_pass:
                # display current progress
                self._ffmpeg_backend.execute(
                    progress_callback=progress_update_cb,
                    pass_num=2,
                )
                progress_complete_cb()

        cmd_mgr = CommandManager(self._input_files, output_dir=self._output_dir, steps=2 if self._two_pass else 1, overwrite=self._overwrite_output)
        cmd_mgr.run(_callback, out_suffix=f".{self._file_format}", out_stem=self._out_stem)

        logger.info(f"{_('FFMpeg result')}: [green][bold]{_('SUCCESS')}[/bold][/green]")
        return self


__all__ = [
    "FFmpegCmdHelper",
    "EXTERNAL_DEPENDENCIES",
]



# src\file_conversor\utils\command_manager.py

from pathlib import Path
from typing import Any, Callable, List

# user-provided
from file_conversor.config.locale import get_translation

from file_conversor.utils.progress_manager import ProgressManager

_ = get_translation()


class CommandManager:
    @staticmethod
    def get_output_file(
            output_file: str | Path,
            stem: str | None = None,
            suffix: str | None = None,
    ):
        """
        Get output file based on input
        """
        output_file = Path(output_file)
        output_name = output_file.with_stem(f"{output_file.with_suffix("").name}{stem or ""}")
        output_name = output_name.with_suffix(suffix if suffix is not None else output_file.suffix)
        return Path(output_name.name)

    def __init__(self, input_files: List[str] | List[Path] | str | Path, output_dir: Path, overwrite: bool, steps: int = 1) -> None:
        super().__init__()
        self._overwrite = overwrite
        self._output_dir = output_dir
        self._steps = steps
        self._input_files: list[Path] = []

        if isinstance(input_files, list):
            self._input_files.extend([Path(f) for f in input_files])
        else:
            self._input_files.append(Path(input_files))

    def run(self, callback: Callable[[Path, Path, ProgressManager], Any], out_stem: str | None = None, out_suffix: str | None = None):
        """
        Run batch command

        :param callback: lambda input_file, output_file, progress_mgr
        """
        with ProgressManager(len(self._input_files), total_steps_per_file=self._steps) as progress_mgr:
            for input_file in self._input_files:
                output_file = self._output_dir / self.get_output_file(input_file, stem=out_stem, suffix=out_suffix)
                if not self._overwrite and output_file.exists():
                    raise FileExistsError(f"{_("File")} '{output_file}' {_("exists")}. {_("Use")} 'file_conversor -oo' {_("to overwrite")}.")

                callback(input_file, output_file, progress_mgr)


__all__ = [
    "CommandManager",
]



# src\file_conversor\backend\gui\_components\figure.py

from file_conversor.utils.dominate_utils import figure, img


def Figure(
        _src: str,
        _alt: str = '',
        _class="",
        **kwargs,
):
    """
    Create a figure element with an image.

    :param _src: The source URL of the image.
    :param _alt: The alt text for the image.
    :param _class: Additional CSS classes for the figure.
    """
    with figure(_class=f"image {_class}", **kwargs) as fig:
        img(
            src=_src,
            alt=_alt,
        )
    return fig


__all__ = [
    'Figure',
]



# src\file_conversor\backend\audio_video\format_container.py

from pathlib import Path
from typing import Any, Iterable, Self

# user-provided imports
from file_conversor.config import Log
from file_conversor.config.locale import get_translation

from file_conversor.backend.audio_video.ffmpeg_codec import _FFmpegCodec, FFmpegAudioCodec, FFmpegVideoCodec

from file_conversor.utils import AbstractRegisterManager

_ = get_translation()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class FormatContainer(AbstractRegisterManager):
    @staticmethod
    def _check_available_codec(codec: _FFmpegCodec, available_codecs: Iterable[str]):
        if codec.name in available_codecs:
            return codec
        raise ValueError(f"Codec '{codec}' {_('not available. Available codecs are:')} {', '.join(available_codecs)}")

    def __init__(
        self,
        name: str,
        audio_codec: str,
        video_codec: str,
        available_audio_codecs: set[str],
        available_video_codecs: set[str],

    ) -> None:
        super().__init__()
        self._name = name
        self._available_audio_codecs = available_audio_codecs
        self._available_video_codecs = available_video_codecs

        self.audio_codec = FFmpegAudioCodec.from_str(audio_codec)
        self.video_codec = FFmpegVideoCodec.from_str(video_codec)

    # PROPERTIES
    @property
    def available_audio_codecs(self):
        return self._available_audio_codecs.copy()

    @property
    def available_video_codecs(self):
        return self._available_video_codecs.copy()

    @property
    def audio_codec(self):
        return self._audio_codec

    @audio_codec.setter
    def audio_codec(self, value):
        if not isinstance(value, FFmpegAudioCodec):
            raise ValueError(f"Cannot set '{type(value)}({value})' as audio codec.")
        self._check_available_codec(value, self._available_audio_codecs)
        self._audio_codec = value

    @property
    def video_codec(self):
        return self._video_codec

    @video_codec.setter
    def video_codec(self, value):
        if not isinstance(value, FFmpegVideoCodec):
            raise ValueError(f"Cannot set '{type(value)}({value})' as video codec.")
        self._check_available_codec(value, self._available_video_codecs)
        self._video_codec = value

    # METHODS
    def __eq__(self, value: object) -> bool:
        if isinstance(value, FormatContainer):
            return (self._name == value._name and
                    self.audio_codec == value.audio_codec and
                    self.video_codec == value.video_codec)
        return False

    def __hash__(self) -> int:
        return hash(self._name)

    def __repr__(self) -> str:
        return f"{self._name} (audio={self.audio_codec}, video={self.video_codec})"

    def __str__(self) -> str:
        return self._name

    def get_options(self) -> list[str]:
        res = ["-f", self._name]
        if self._name.lower() != "null":
            res.extend(self.audio_codec.get_options())
            res.extend(self.video_codec.get_options())
        return res


class VideoFormatContainer(FormatContainer):
    _REGISTERED: dict[str, tuple[tuple, dict[str, Any]]] = {}


class AudioFormatContainer(FormatContainer):
    _REGISTERED: dict[str, tuple[tuple, dict[str, Any]]] = {}


# AUDIO CONTAINERS
AudioFormatContainer.register(
    "null",
    name="null",
    audio_codec="null",
    video_codec="null",
    available_audio_codecs={
        "null", "copy",
    },
    available_video_codecs={
        "null", "copy",
    },
)
AudioFormatContainer.register(
    'mp3',
    name="mp3",
    audio_codec="libmp3lame",
    video_codec="null",
    available_audio_codecs={
        "libmp3lame",
        "copy",
    },
    available_video_codecs={
        "null",
    },
)
AudioFormatContainer.register(
    'm4a',
    name="ipod",
    audio_codec="aac",
    video_codec="null",
    available_audio_codecs={
        "aac",
        "copy",
    },
    available_video_codecs={
        "null",
    },
)
AudioFormatContainer.register(
    'ogg',
    name="ogg",
    audio_codec="libvorbis",
    video_codec="null",
    available_audio_codecs={
        "libvorbis",
        "copy",
    },
    available_video_codecs={
        "null",
    },
)
AudioFormatContainer.register(
    'opus',
    name="opus",
    audio_codec="libopus",
    video_codec="null",
    available_audio_codecs={
        "libopus",
        "copy",
    },
    available_video_codecs={
        "null",
    },
)
AudioFormatContainer.register(
    'flac',
    name="flac",
    audio_codec="flac",
    video_codec="null",
    available_audio_codecs={
        "flac",
        "copy",
    },
    available_video_codecs={
        "null",
    },
)

# VIDEO CONTAINERS
VideoFormatContainer.register(
    "null",
    name="null",
    audio_codec="null",
    video_codec="null",
    available_audio_codecs={
        "null", "copy",
    },
    available_video_codecs={
        "null", "copy",
    },
)
VideoFormatContainer.register(
    'mp4',
    name="mp4",
    audio_codec="aac",
    video_codec="libx264",
    available_audio_codecs={
        "aac",
        "ac3",
        "libmp3lame",
        "null", "copy",
    },
    available_video_codecs={
        "libx264",
        "libx265",

        "h264_nvenc",
        "hevc_nvenc",

        "h264_vaapi",
        "hevc_vaapi",

        "h264_qsv",
        "hevc_qsv",

        "copy",
    },
)
VideoFormatContainer.register(
    'avi',
    name="avi",
    audio_codec="libmp3lame",
    video_codec="mpeg4",
    available_audio_codecs={
        "libmp3lame",
        "pcm_s16le",
        "null", "copy",
    },
    available_video_codecs={
        "mpeg4",
        "copy",
    },
)
VideoFormatContainer.register(
    'mkv',
    name="matroska",
    audio_codec="aac",
    video_codec="libx264",
    available_audio_codecs={
        "aac",
        "ac3",
        "libmp3lame",
        "libopus",
        "libvorbis",
        "flac",
        "null", "copy",
    },
    available_video_codecs={
        "libx264",
        "libx265",

        "h264_nvenc",
        "hevc_nvenc",

        "h264_vaapi",
        "hevc_vaapi",

        "h264_qsv",
        "hevc_qsv",

        "libvpx",
        "libvpx-vp9",
        "libaom-av1",

        "vp8_vaapi",
        "vp9_vaapi",

        "vp8_qsv",
        "vp9_qsv",

        "av1_nvenc",
        "av1_vaapi",
        "av1_qsv",

        "copy",
    },
)
VideoFormatContainer.register(
    'webm',
    name="webm",
    audio_codec="libvorbis",
    video_codec="libvpx",
    available_audio_codecs={
        "libvorbis",
        "libopus",
        "null", "copy",
    },
    available_video_codecs={
        "libvpx",
        "libvpx-vp9",
        "libaom-av1",

        "vp8_vaapi",
        "vp9_vaapi",

        "vp8_qsv",
        "vp9_qsv",

        "av1_nvenc",
        "av1_vaapi",
        "av1_qsv",

        "copy",
    },
)

__all__ = [
    "FormatContainer",
    "VideoFormatContainer",
    "AudioFormatContainer",
]



# src/file_conversor/backend/gui/audio/convert.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.audio_video import FFmpegBackend

from file_conversor.backend.gui._dom_page import home_nav_item
from file_conversor.backend.gui.audio._dom_page import audio_convert_nav_item, audio_index_nav_item

from file_conversor.utils.bulma_utils import AudioBitrateField, FileFormatField, InputFilesField, OutputDirField
from file_conversor.utils.dominate_bulma import PageForm

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def PageAudioConvert():
    return PageForm(
        InputFilesField(
            *[f for f in FFmpegBackend.SUPPORTED_IN_FORMATS],
            description=_("Audio and Video files"),
        ),
        FileFormatField(*[
            (q, q.upper())
            for q in filter(lambda x: x.lower() != 'null', FFmpegBackend.SUPPORTED_OUT_AUDIO_FORMATS)
        ], current_value='mp3'),
        OutputDirField(),
        AudioBitrateField(),
        api_endpoint=url_for('api_audio_convert'),
        nav_items=[
            home_nav_item(),
            audio_index_nav_item(),
            audio_convert_nav_item(active=True),
        ],
        _title=f"{_('Audio Convert')} - File Conversor",
    )


def audio_convert():
    return render_template_string(str(
        PageAudioConvert()
    ))


__all__ = ['audio_convert']



# src\file_conversor\cli\text\_typer.py

# user-provided modules
from file_conversor.config import get_translation

_ = get_translation()

# command
COMMAND_NAME = "text"

# SUBCOMMANDS
CHECK_NAME = "check"
COMPRESS_NAME = "compress"
CONVERT_NAME = "convert"

__all__ = [
    "COMMAND_NAME",

    "CHECK_NAME",
    "COMPRESS_NAME",
    "CONVERT_NAME",
]



# src/file_conversor/backend/gui/config/_tab_general.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.utils.dominate_bulma import FormFieldCheckbox, FormFieldHorizontal, FormFieldInput, FormFieldSelect
from file_conversor.utils.bulma_utils import OutputDirField

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import AVAILABLE_LANGUAGES, get_translation, get_language_name

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def TabConfigGeneral() -> tuple | list:
    languages = [
        (k, get_language_name(k))
        for k in AVAILABLE_LANGUAGES
    ]
    languages.sort(key=lambda x: x[1])  # sort by language name

    return (
        OutputDirField(
            _name="gui-output-dir",
            help=_("Select the default output directory for saving converted files."),
            label_text=_("Output Directory"),
        ),
        FormFieldHorizontal(
            FormFieldSelect(
                *languages,
                current_value=CONFIG['language'],
                _name="language",
                help=_("Select the desired language for the user interface."),
            ),
            label_text=_("Language"),
        ),
        FormFieldHorizontal(
            FormFieldInput(
                validation_expr="Number.parseInt(value) >= 1",
                current_value=CONFIG['gui-zoom'],
                _name="gui-zoom",
                _type="number",
                help=_("Set the default zoom level for the user interface. Valid values are >= 1 (100 = normal size, 150 = 1.5x size, etc)."),
                x_init="""
                    let timeout;
                    this.$watch('value', (newValue) => {
                        clearTimeout(timeout);
                        timeout = setTimeout(() => {
                            set_zoom(newValue);
                        }, 1500);
                    });
                """,

            ),
            label_text=_("GUI Zoom Level (%)"),
        ),
        FormFieldHorizontal(
            FormFieldInput(
                validation_expr="Number.parseInt(value) >= 60",
                current_value=CONFIG['cache-expire-after'],
                _name="cache-expire-after",
                _type="number",
                help=_("HTTP cache expiration time in seconds (e.g., 1h = 3600 secs, 1d = 86400 secs). Minimum of 60 seconds."),
            ),
            label_text=_("HTTP Cache Expiration (secs)"),
        ),
        FormFieldCheckbox(
            current_value=CONFIG['cache-enabled'],
            _name="cache-enabled",
            label_text=_('Enable HTTP cache.'),
            help=_('If enabled, the application will use HTTP cache to improve performance.'),
        ),
        FormFieldCheckbox(
            current_value=CONFIG['install-deps'],
            _name="install-deps",
            label_text=_('Automatic install of missing dependencies, on demand.'),
            help=_('If enabled, the application will attempt to automatically install any missing dependencies when needed.'),
        ),
    )


__all__ = ['TabConfigGeneral']



# src\file_conversor\backend\gui\_components\status_bar.py

from file_conversor.utils.dominate_bulma.progress import Progress
from file_conversor.utils.dominate_utils import div, span


def StatusBar(**kwargs):
    with div(
        _id="status_bar",
        _class="status-bar is-fixed-bottom is-flex is-justify-content-space-between "
               "is-align-items-center is-size-6 p-1 pl-3 pr-3",
        **{"x-data": "{ }"},
        **kwargs
    ) as status_bar:

        # Left/start section (status message)
        with div(_class="status-bar-start is-flex is-align-items-center"):
            span(
                "STATUS:",
                _class="has-text-weight-bold mr-2",
                **{
                    ":class": """{
                        'is-flex': $store.status_bar.message !== '', 
                        'is-hidden': $store.status_bar.message === '',
                    }"""
                }
            )
            span("", _class="status-message", **{"x-text": "$store.status_bar.message"})

        # Center section (progress)
        with div(
            _class="status-bar-center is-align-items-center is-flex-grow-1 mx-6 px-6",
            **{
                ":class": """{
                    'is-flex': (typeof $store.status_bar.progress === 'number'), 
                    'is-hidden': (typeof $store.status_bar.progress !== 'number'),
                }"""
            }
        ):
            Progress(_class="status-progress-bar is-small is-success my-0 mr-3")
            span(
                _class="status-progress-text",
                **{
                    ":class": """{
                        'is-hidden': (typeof $store.status_bar.progress !== 'number' || $store.status_bar.progress < 0)
                    }""",
                    "x-text": "`${Math.round($store.status_bar.progress)}%`"
                }
            )

        # Right/end section (time)
        with div(
            _class="status-bar-end is-align-items-center",
            **{
                ":class": """{
                    'is-flex': (typeof $store.status_bar.time === 'number'), 
                    'is-hidden': (typeof $store.status_bar.time !== 'number'),
                }"""
            }
        ):
            span(
                _class="status-time",
                **{
                    "x-text": "new Date($store.status_bar.time * 1000).toISOString().substring(11, 19)"
                }
            )
    return status_bar


__all__ = ["StatusBar"]



# src/file_conversor/backend/gui/xls/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui.xls._index import xls_index

from file_conversor.backend.gui.xls.convert import xls_convert


def routes():
    return [
        FlaskRoute(
            rule="/xls",
            handler=xls_index
        ),
        # TOOLS
        FlaskRoute(
            rule="/xls/convert",
            handler=xls_convert
        ),
    ]


__all__ = [
    'xls_index',
    'xls_convert',

    'routes',
]



# src\file_conversor\backend\audio_video\ffmpeg_backend.py

"""
This module provides functionalities for handling audio and video files using FFmpeg.
"""

import subprocess
import re

from pathlib import Path
from typing import Any, Callable, Iterable

from cv2 import log

# user-provided imports
from file_conversor.backend.audio_video.abstract_ffmpeg_backend import AbstractFFmpegBackend
from file_conversor.backend.audio_video.ffprobe_backend import FFprobeBackend

from file_conversor.backend.audio_video.ffmpeg_filter import FFmpegFilter
from file_conversor.backend.audio_video.ffmpeg_codec import FFmpegAudioCodec, FFmpegVideoCodec
from file_conversor.backend.audio_video.format_container import FormatContainer

from file_conversor.config import Environment, Log, get_translation

from file_conversor.utils.validators import check_file_format
from file_conversor.utils.command_manager import CommandManager

from file_conversor.system import is_windows

_ = get_translation()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class FFmpegBackend(AbstractFFmpegBackend):
    """
    FFmpegBackend is a class that provides an interface for handling audio and video files using FFmpeg.
    """

    EXTERNAL_DEPENDENCIES: set[str] = {
        "ffmpeg",
    }

    @staticmethod
    def _clean_two_pass_log_file(logfile: Path | None):
        if logfile is None:
            return
        for filepath in logfile.parent.glob(logfile.name + "-0.log*"):
            try:
                if not filepath.exists():
                    continue
                filepath.unlink()
            except Exception as e:
                logger.warning(f"Failed to remove log file '{filepath}': {e}")

    def __init__(
        self,
        install_deps: bool | None,
        verbose: bool = False,
        overwrite_output: bool = False,
        stats: bool = False,
    ):
        """
        Initialize the FFMpeg backend.

        :param install_deps: Install external dependencies. If True auto install using a package manager. If False, do not install external dependencies. If None, asks user for action. 
        :param verbose: Verbose logging. Defaults to False.      

        :raises RuntimeError: if ffmpeg dependency is not found
        """
        super().__init__(
            install_deps=install_deps,
            verbose=verbose,
        )

        self._global_options: list[str] = [
            "" if not verbose else "-v",
            "" if not stats else "-stats",
            "-n" if not overwrite_output else "-y",
        ]
        self._in_opts: list[str] = []
        self._out_opts: list[str] = []

        self._out_container: FormatContainer | None = None

        self._input_file: Path | None = None
        self._output_file: Path | None = None
        self._pass_logfile: Path | None = None

        self._audio_bitrate: int | None = None
        self._video_bitrate: int | None = None

        self._progress_callback: Callable[[float], Any] | None = None

    def _execute_progress_callback(
        self,
        process: subprocess.Popen,
    ):
        """returns output lines read"""
        lines: list[str] = []
        PROGRESS_RE = re.compile(r'time=(\d+):(\d+):([\d\.]+)')

        ffprobe_backend = FFprobeBackend(install_deps=self._install_deps, verbose=self._verbose)
        if not self._input_file:
            raise RuntimeError(f"{_('Input file not set')}")

        file_duration_secs = ffprobe_backend.get_duration(self._input_file)
        while process.poll() is None:
            if not process.stdout:
                continue

            line = process.stdout.readline()
            match = PROGRESS_RE.search(line)
            if not match:
                lines.append(line)
                continue
            hours = int(match.group(1))
            minutes = int(match.group(2))
            seconds = float(match.group(3))

            current_time = hours * 3600 + minutes * 60 + seconds
            progress = 100.0 * (float(current_time) / file_duration_secs)
            if self._progress_callback:
                self._progress_callback(progress)
        return lines

    def _set_input_file(self, input_file: str | Path):
        """
        Set the input file and check if it has a supported format.

        :param input_file: Input file path.

        :raises FileNotFoundError: If the input file does not exist.
        :raises ValueError: If the input file format is not supported.
        """
        self._in_opts = []

        # check file is found
        self._input_file = Path(input_file).resolve()
        if not self._input_file.exists() and not self._input_file.is_file():
            raise FileNotFoundError(f"Input file '{input_file}' not found")

        # check if the input file has a supported format
        check_file_format(self._input_file, self.SUPPORTED_IN_FORMATS)

        # set the input format options based on the file extension
        in_ext = self._input_file.suffix[1:].lower()
        for k, v in self.SUPPORTED_IN_FORMATS[in_ext].items():
            self._in_opts.extend([str(k), str(v)])

    def _set_output_file(
            self,
            output_file: str | Path,
    ):
        """
        Set the output file and check if it has a supported format.

        :param output_file: Output file path.

        :raises typer.BadParameter: Unsupported format.
        """
        self._out_opts = []

        # create out dir (if it does not exists)
        self._output_file = Path(output_file).resolve()
        self._output_file = self._output_file.with_suffix(self._output_file.suffix.lower())

        if self._output_file.name == "-":
            logger.warning("Null container selected. No output file will be created.")
            out_ext = "null"
        else:
            self._output_file.parent.mkdir(parents=True, exist_ok=True)

            # check if the output file has a supported format
            check_file_format(self._output_file, self.SUPPORTED_OUT_FORMATS)

            # set the output format options based on the file extension
            out_ext = self._output_file.suffix[1:]
        args, kwargs = self.SUPPORTED_OUT_FORMATS[out_ext]
        self._out_container = FormatContainer(*args, **kwargs)
        self._set_pass_logfile()

    def _set_pass_logfile(self):
        if not self._output_file:
            raise RuntimeError(f"{_('Output file not set')}")

        logdir = self._output_file.parent
        self._pass_logfile = logdir / CommandManager.get_output_file(self._output_file, stem="-ffmpeg2pass", suffix="")
        logger.debug(f"{_('Temporary 2-pass log file')}: {self._pass_logfile}")

    def set_files(self, input_file: str | Path, output_file: str | Path):
        """
        Set input/output files, and default global options

        :param input_file: Input file path.
        :param output_file: Output file path.      
        """
        self._set_input_file(input_file)
        self._set_output_file(output_file)

    def _set_codec(
            self,
            is_audio: bool,
            codec: str | None = None,
            bitrate: int | None = None,
            filters: FFmpegFilter | Iterable[FFmpegFilter] | None = None,
    ):
        if not self._out_container:
            raise RuntimeError(f"{_('Output container not set')}")

        if codec:
            if is_audio:
                self._out_container.audio_codec = FFmpegAudioCodec.from_str(codec)
            else:
                self._out_container.video_codec = FFmpegVideoCodec.from_str(codec)

        codec_obj = self._out_container.audio_codec if is_audio else self._out_container.video_codec

        if bitrate is not None and bitrate > 0:
            if is_audio:
                self._audio_bitrate = bitrate
            else:
                self._video_bitrate = bitrate
            codec_obj.set_bitrate(bitrate)

        if filters:
            if isinstance(filters, FFmpegFilter):
                filters = [filters]
            codec_obj.set_filters(*filters)

    def set_audio_codec(
        self,
        codec: str | None = None,
        bitrate: int | None = None,
        filters: FFmpegFilter | Iterable[FFmpegFilter] | None = None,
    ):
        """
        Set audio codec and bitrate

        :param codec: Codec to use. Defaults to None (use container default codec).      
        :param bitrate: Bitrate to use (in kbps). Defaults to None (use FFmpeg defaults).      
        :param filters: Filters to use. Defaults to None (do not use any filter).      

        :raises RuntimeErrors: if output container not set
        """
        self._set_codec(is_audio=True, codec=codec, bitrate=bitrate, filters=filters)

    def set_video_codec(
        self,
        codec: str | None = None,
        bitrate: int | None = None,
        filters: FFmpegFilter | Iterable[FFmpegFilter] | None = None,
        encoding_speed: str | None = None,
        quality_setting: str | None = None,
    ):
        """
        Seet video codec and bitrate

        :param codec: Codec to use. Defaults to None (use container default codec).      
        :param bitrate: Bitrate to use (in kbps). Defaults to None (use FFmpeg defaults).      
        :param filters: Filters to use. Defaults to None (do not use any filter).      
        :param encoding_speed: Encoding speed to use. Defaults to None (use codec default speed).      
        :param quality_setting: Quality setting to use. Defaults to None (use codec default quality).

        :raises RuntimeErrors: if output container not set
        """
        self._set_codec(is_audio=False, codec=codec, bitrate=bitrate, filters=filters)
        if not self._out_container:
            raise RuntimeError(f"{_('Output container not set')}")

        if encoding_speed:
            self._out_container.video_codec.set_encoding_speed(encoding_speed)

        if (not bitrate or bitrate == 0) and quality_setting:
            self._out_container.video_codec.set_quality_setting(quality_setting)

    def _get_two_pass_options(self, pass_num: int) -> list[str]:
        if pass_num <= 0:
            return []

        if self._audio_bitrate and self._audio_bitrate <= 0:
            raise ValueError(f"{_('Audio Bitrate cannot be 0 when using two-pass mode.')}")
        if self._video_bitrate and self._video_bitrate <= 0:
            raise ValueError(f"{_('Video Bitrate cannot be 0 when using two-pass mode.')}")
        if not self._pass_logfile:
            raise RuntimeError(f"{_('2-pass log file not set')}")

        # add 2-pass encoding options
        return [
            "-pass", str(pass_num),
            "-passlogfile", str(self._pass_logfile),
        ]

    def _execute(self):
        # build ffmpeg command
        ffmpeg_command = []
        ffmpeg_command.extend([str(self._ffmpeg_bin)])  # ffmpeg CLI
        ffmpeg_command.extend(self._global_options)    # set global options
        ffmpeg_command.extend(self._in_opts)           # set in options
        ffmpeg_command.extend(["-i", str(self._input_file)])   # set input
        ffmpeg_command.extend(self._out_opts)          # set out options
        ffmpeg_command.extend([str(self._output_file)])        # set output

        # remove empty strings
        ffmpeg_command = [arg for arg in ffmpeg_command if arg != ""]

        # Execute the FFmpeg command
        process = Environment.run_nowait(
            *ffmpeg_command,
        )

        out_lines = self._execute_progress_callback(
            process=process,
        )

        Environment.check_returncode(process, out_lines=out_lines)
        return process

    def execute(
        self,
        progress_callback: Callable[[float], Any] | None = None,
        pass_num: int = 0,
        out_opts: list[str] | None = None,
    ):
        """
        Execute the FFmpeg command to convert the input file to the output file.

        :param pass_num: Pass number for multi-pass encoding (0 for single pass, 1 for first pass, 2 for second pass). Defaults to 0.
        :param out_opts: FFmpeg custom out options. Defaults to None.

        :return: Subprocess.Popen object

        :raises RuntimeError: If FFmpeg encounters an error during execution.
        """
        self._progress_callback = progress_callback

        if pass_num not in (0, 1, 2):
            raise ValueError(f"{_('Invalid number of passes:')} {pass_num}. {_('Must be 0 (single-pass), 1 (first pass) or 2 (second pass).')}")

        if not self._input_file or not self._output_file:
            raise RuntimeError(f"{_('Input/output files not set')}")

        if not self._out_container:
            raise RuntimeError(f"{_('Output container not set')}")

        self._out_opts = [
            *self._get_two_pass_options(pass_num),
            *self._out_container.get_options(),
            *(out_opts or []),
        ]

        original_output_file = self._output_file
        if pass_num == 1:
            self._output_file = Path("NUL" if is_windows() else "/dev/null")

        try:
            self._execute()
        except:
            self._clean_two_pass_log_file(self._pass_logfile)
            raise

        self._output_file = original_output_file

        if pass_num in (0, 2):
            self._clean_two_pass_log_file(self._pass_logfile)


__all__ = [
    "FFmpegBackend",
]



# src/file_conversor/backend/gui/pdf/encrypt.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.pdf import PyPDFBackend

from file_conversor.backend.gui._dom_page import home_nav_item
from file_conversor.backend.gui.pdf._dom_page import pdf_encrypt_nav_item, pdf_index_nav_item

from file_conversor.utils.bulma_utils import InputFilesField, OutputDirField, PDFEncryptionAlgorithmField, PDFPasswordField
from file_conversor.utils.dominate_bulma import SmartGrid, Cell, FormFieldCheckbox, Tabs, PageForm

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def TabGeneral() -> list | tuple:
    return [
        InputFilesField(
            *PyPDFBackend.SUPPORTED_IN_FORMATS,
            description=_("PDF files"),
        ),
        PDFEncryptionAlgorithmField(),
        PDFPasswordField(
            _validation_expr="value.length > 0",
            _name="owner-password",
            help=_("Owner password used to encrypt the PDF files. Owner has ALL THE PERMISSIONS."),
            label_text=_("Owner Password"),
        ),
        OutputDirField(),
    ]


def TabAdvanced() -> list | tuple:
    return [
        PDFPasswordField(
            _name="user-password",
            help=_("User password used to encrypt the PDF files. User has ONLY THE PERMISSIONS specified below. Leave blank if not needed."),
            label_text=_("User Password"),
        ),
        SmartGrid(
            Cell(
                FormFieldCheckbox(
                    _name="annotate",
                    current_value="off",
                    help=_("Allow annotations (comments, highlight text, etc) in the encrypted PDF files."),
                    label_text=_("Allow annotations"),
                    reverse=True,
                ),
            ),
            Cell(
                FormFieldCheckbox(
                    _name="fill_forms",
                    current_value="off",
                    help=_("Allow fill forms in the encrypted PDF files."),
                    label_text=_("Allow fill forms"),
                    reverse=True,
                ),
            ),
            Cell(
                FormFieldCheckbox(
                    _name="modify",
                    current_value="off",
                    help=_("Allow encrypted PDF modifications."),
                    label_text=_("Allow modify"),
                    reverse=True,
                ),
            ),
            Cell(
                FormFieldCheckbox(
                    _name="modify_pages",
                    current_value="off",
                    help=_("Allow modifying pages (add, delete, rotate, reorder) in the encrypted PDF files."),
                    label_text=_("Allow modify pages"),
                    reverse=True,
                ),
            ),
            Cell(
                FormFieldCheckbox(
                    _name="copy",
                    current_value="off",
                    help=_("Allow copying content from the encrypted PDF files."),
                    label_text=_("Allow copy"),
                    reverse=True,
                ),
            ),
            Cell(
                FormFieldCheckbox(
                    _name="accessibility",
                    current_value="on",
                    help=_("Allow accessibility features (screen readers, etc) in the encrypted PDF files."),
                    label_text=_("Allow accessibility features"),
                    reverse=True,
                ),
            ),
            Cell(
                FormFieldCheckbox(
                    _name="print_lq",
                    current_value="on",
                    help=_("Allow encrypted PDF printing (low quality)."),
                    label_text=_("Allow print low quality"),
                    reverse=True,
                ),
            ), Cell(
                FormFieldCheckbox(
                    _name="print_hq",
                    current_value="on",
                    help=_("Allow encrypted PDF printing (high quality)."),
                    label_text=_("Allow print high quality"),
                    reverse=True,
                ),
            ),
            _class="is-col-min-12 is-gap-2",
        ),
    ]


def PagePDFEncrypt():
    return PageForm(
        Tabs(
            {
                'label': _('General'),
                'icon': 'cog',
                'content': TabGeneral(),
            },
            {
                'label': _('Advanced'),
                'icon': 'tools',
                'content': TabAdvanced(),
            },
            active_tab=_('General'),
            _class="""
                is-toggle 
                is-toggle-rounded 
                is-flex 
                is-full-width 
                is-flex-direction-column 
                is-align-items-center
                mb-4
            """,
            _class_headers="mb-4",
            _class_content="is-full-width",
        ),
        api_endpoint=f"{url_for('api_pdf_encrypt')}",
        nav_items=[
            home_nav_item(),
            pdf_index_nav_item(),
            pdf_encrypt_nav_item(active=True),
        ],
        _title=f"{_('PDF Encrypt')} - File Conversor",
    )


def pdf_encrypt():
    return render_template_string(str(
        PagePDFEncrypt()
    ))



# src\file_conversor\backend\gui\_components\breadcrumb.py

from typing import Any

from file_conversor.utils.dominate_bulma.font_awesome_icon import FontAwesomeIcon
from file_conversor.utils.dominate_utils import a, li, nav, ul


def Breadcrumb(*items: dict[str, Any], _class: str = "", **kwargs):
    """
    Create a breadcrumb navigation component.

    :param items: A list of breadcrumb items, each represented as a dictionary with keys.
    :param _class: Additional CSS classes to apply to the breadcrumb container.
    :param kwargs: Additional attributes for the breadcrumb container.

    ```python
    items = [{
        'icon': (Optional) The icon name to display alongside the label
        'label': The display text for the item,
        'url': The URL the item links to,
        'active': A boolean indicating if the item is the current page,
    }]
    ```
    """
    with nav(_class=f"breadcrumb {_class}", _aria_label="breadcrumbs", **kwargs) as component:
        with ul():
            for item in items:
                with li(_class="is-active" if item.get("active") else ""):
                    if item.get("icon"):
                        FontAwesomeIcon(item["icon"])
                    a(item["label"], _href=item["url"])
    return component


__all__ = ["Breadcrumb"]




# src\file_conversor\cli\multimedia\image_cmd.py

import typer

from pathlib import Path
from typing import Annotated, Any, Callable, List

from rich import print

# user-provided modules
from file_conversor.backend.image import PillowBackend

from file_conversor.cli.image._typer import TRANSFORMATION_PANEL as RICH_HELP_PANEL
from file_conversor.cli.image._typer import COMMAND_NAME, RESIZE_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager

from file_conversor.utils.formatters import parse_image_resize_scale
from file_conversor.utils.typer_utils import InputFilesArgument, OutputDirOption
from file_conversor.utils.validators import check_positive_integer, check_valid_options

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = PillowBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    # Pillow commands
    for ext in PillowBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="resize",
                description="Resize",
                command=f'cmd.exe /k "{Environment.get_executable()} "{COMMAND_NAME}" "{RESIZE_NAME}" "%1""',
                icon=str(icons_folder_path / "resize.ico"),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_image_resize_cmd(
    input_files: List[Path],
    scale: float | None,
    width: int | None,
    resampling: str,
    output_dir: Path,
    progress_callback: Callable[[float], Any] = lambda p: p,
):
    pillow_backend = PillowBackend(verbose=STATE['verbose'])

    scale = parse_image_resize_scale(scale, width, quiet=STATE["quiet"])

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        logger.info(f"Processing '{output_file}' ... ")
        pillow_backend.resize(
            input_file=input_file,
            output_file=output_file,
            scale=scale,
            width=width,
            resampling=PillowBackend.RESAMPLING_OPTIONS[resampling],
        )
        progress_callback(progress_mgr.complete_step())

    cmd_mgr = CommandManager(input_files, output_dir=output_dir, overwrite=STATE["overwrite-output"])
    cmd_mgr.run(callback, out_stem="_resized")

    logger.info(f"{_('Image resize')}: [green][bold]{_('SUCCESS')}[/bold][/green]")


@typer_cmd.command(
    name=RESIZE_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Resize an image file.')}

        {_('Outputs an image file with _resized at the end.')}
""",
    epilog=f"""
        **{_('Examples')}:**



        *{_('Double the image size')}*:

        - `file_conversor {COMMAND_NAME} {RESIZE_NAME} input_file.jpg -s 2.0`



        *{_('Set the image width to 1024px')}*:

        - `file_conversor {COMMAND_NAME} {RESIZE_NAME} input_file.jpg -od D:/Downloads -w 1024`
    """)
def resize(
    input_files: Annotated[List[Path], InputFilesArgument(PillowBackend)],
    scale: Annotated[float | None, typer.Option("--scale", "-s",
                                                help=f"{_("Scale image proportion. Valid values start at 0.1. Defaults to")} None (use width to scale image).",
                                                callback=lambda x: check_positive_integer(x),
                                                )] = None,

    width: Annotated[int | None, typer.Option("--width", "-w",
                                              help=f"{_("Width in pixels (height is calculated based on width to keep image proportions). Defaults to")} None ({_('use scale to resize image')}).",
                                              callback=lambda x: check_positive_integer(x),
                                              )] = None,

    resampling: Annotated[str, typer.Option("--resampling", "-r",
                                            help=f'{_("Resampling algorithm. Valid values are")} {", ".join(PillowBackend.RESAMPLING_OPTIONS)}. {_("Defaults to")} {CONFIG["image-resampling"]}.',
                                            callback=lambda x: check_valid_options(x, PillowBackend.RESAMPLING_OPTIONS),
                                            )] = CONFIG["image-resampling"],
    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    execute_image_resize_cmd(
        input_files=input_files,
        scale=scale,
        width=width,
        resampling=resampling,
        output_dir=output_dir,
    )


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
    "execute_image_resize_cmd",
]



# src/file_conversor/backend/gui/_api/config.py

from flask import json, render_template, url_for

from typing import Any

# user-provided modules
from file_conversor.backend.gui.flask_api import FlaskApi, FlaskApiStatus

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def config_thread(params: dict[str, Any], status: FlaskApiStatus) -> None:
    """API endpoint to update the application configuration."""
    logger.info(f"[bold]{_('Configuration update requested via API.')}[/]")
    CONFIG.update(params)
    CONFIG.save()
    logger.debug(f"Configuration updated: {CONFIG}")


def api_config():
    """API endpoint to update the application configuration."""
    logger.info(f"[bold]{_('Configuration set requested via API.')}[/]")
    return FlaskApi.execute_response(config_thread)



# src\file_conversor\cli\_info.py

import sys
import typer

from typing import Any

# user-provided modules
from file_conversor.config import get_translation
from file_conversor.system import is_windows

# CLI
from file_conversor.cli.audio import audio_cmd
from file_conversor.cli.video import video_cmd
from file_conversor.cli.config import config_cmd
from file_conversor.cli.doc import doc_cmd
from file_conversor.cli.ebook import ebook_cmd
from file_conversor.cli.gui import gui_cmd
from file_conversor.cli.hash import hash_cmd
from file_conversor.cli.image import image_cmd
from file_conversor.cli.pdf import pdf_cmd
from file_conversor.cli.pipeline import pipeline_cmd
from file_conversor.cli.ppt import ppt_cmd
from file_conversor.cli.text import text_cmd
from file_conversor.cli.win import win_cmd
from file_conversor.cli.xls import xls_cmd

_ = get_translation()

PYTHON_VERSION = f"{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}"

# PANELS
OFFICE_PANEL = _("Office files")
FILE_PANEL = _("Other files")
UTILS_CONFIG_PANEL = _("Utils and Config")

# COMMANDS
COMMANDS_LIST: list[dict[str, Any]] = []

################
# OFFICE PANEL #
################
COMMANDS_LIST.extend([
    {
        "typer_instance": doc_cmd,
        "rich_help_panel": OFFICE_PANEL,
    },
    {
        "typer_instance": xls_cmd,
        "rich_help_panel": OFFICE_PANEL,
    },
    {
        "typer_instance": ppt_cmd,
        "rich_help_panel": OFFICE_PANEL,
    },
])

################
# FILE PANEL #
################
COMMANDS_LIST.extend([
    {
        "typer_instance": audio_cmd,
        "rich_help_panel": FILE_PANEL
    },
    {
        "typer_instance": video_cmd,
        "rich_help_panel": FILE_PANEL
    },
    {

        "typer_instance": image_cmd,
        "rich_help_panel": FILE_PANEL
    },
    {

        "typer_instance": pdf_cmd,
        "rich_help_panel": FILE_PANEL
    },
    {

        "typer_instance": ebook_cmd,
        "rich_help_panel": FILE_PANEL
    },
    {
        "typer_instance": text_cmd,
        "rich_help_panel": FILE_PANEL
    },
    {
        "typer_instance": hash_cmd,
        "rich_help_panel": FILE_PANEL
    },
])


######################
# UTILS/CONFIG PANEL
######################
if is_windows():
    COMMANDS_LIST.append({
        "typer_instance": win_cmd,
        "rich_help_panel": UTILS_CONFIG_PANEL
    })

COMMANDS_LIST.extend([
    {
        "typer_instance": config_cmd,
        "rich_help_panel": UTILS_CONFIG_PANEL,
    },
    {
        "typer_instance": pipeline_cmd,
        "rich_help_panel": UTILS_CONFIG_PANEL,
    },
    {
        "typer_instance": gui_cmd,
        "rich_help_panel": UTILS_CONFIG_PANEL,
    },
])

__all__ = [
    "PYTHON_VERSION",
    "COMMANDS_LIST",
]



# src/file_conversor/backend/gui/doc/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui.doc._index import doc_index

from file_conversor.backend.gui.doc.convert import doc_convert


def routes():
    return [
        FlaskRoute(
            rule="/doc",
            handler=doc_index
        ),
        FlaskRoute(
            rule="/doc/convert",
            handler=doc_convert
        ),
    ]


__all__ = [
    'doc_index',
    'doc_convert',

    'routes',
]




# src\file_conversor\cli\video\resize_cmd.py

import typer

from rich import print

from typing import Annotated, List
from pathlib import Path

# user-provided modules
from file_conversor.backend import FFmpegBackend

from file_conversor.cli.video._ffmpeg_cmd_helper import FFmpegCmdHelper, EXTERNAL_DEPENDENCIES

from file_conversor.cli.video._typer import TRANSFORMATION_PANEL as RICH_HELP_PANEL
from file_conversor.cli.video._typer import COMMAND_NAME, RESIZE_NAME

from file_conversor.config import Environment, Configuration, State, Log, get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.validators import check_positive_integer, check_video_resolution
from file_conversor.utils.typer_utils import AudioBitrateOption, FormatOption, InputFilesArgument, OutputDirOption, ResolutionOption, VideoBitrateOption, VideoEncodingSpeedOption, VideoQualityOption

from file_conversor.system.win import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()


def register_ctx_menu(ctx_menu: WinContextMenu):
    # FFMPEG commands
    icons_folder_path = Environment.get_icons_folder()
    for ext in FFmpegBackend.SUPPORTED_IN_VIDEO_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="resize",
                description="Resize",
                command=f'cmd.exe /k "{Environment.get_executable()} "{COMMAND_NAME}" "{RESIZE_NAME}" "%1""',
                icon=str(icons_folder_path / "resize.ico"),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


@typer_cmd.command(
    name=RESIZE_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Resize a video file (downscaling / upscaling).')}

        {_('Outputs a video file with _resized at the end.')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor {COMMAND_NAME} {RESIZE_NAME} input_file.webm -rs 1024:768 -od output_dir/ -f mp4 --audio-bitrate 192`

        - `file_conversor {COMMAND_NAME} {RESIZE_NAME} input_file.mp4 -rs 1280:720`
    """)
def resize(
    input_files: Annotated[List[Path], InputFilesArgument(FFmpegBackend.SUPPORTED_IN_VIDEO_FORMATS)],

    resolution: Annotated[str, ResolutionOption(prompt=f"{_('Enter target resolution (WIDTH:HEIGHT)')}")],

    file_format: Annotated[str, FormatOption(FFmpegBackend.SUPPORTED_OUT_VIDEO_FORMATS)] = CONFIG["video-format"],

    audio_bitrate: Annotated[int, AudioBitrateOption()] = CONFIG["audio-bitrate"],
    video_bitrate: Annotated[int, VideoBitrateOption()] = CONFIG["video-bitrate"],

    video_encoding_speed: Annotated[str | None, VideoEncodingSpeedOption(FFmpegBackend.ENCODING_SPEEDS)] = CONFIG["video-encoding-speed"],
    video_quality: Annotated[str | None, VideoQualityOption(FFmpegBackend.QUALITY_PRESETS)] = CONFIG["video-quality"],

    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):

    ffmpeg_cmd_helper = FFmpegCmdHelper(
        install_deps=CONFIG['install-deps'],
        verbose=STATE["verbose"],
        overwrite_output=STATE["overwrite-output"],
    )

    # Set arguments for FFmpeg command helper
    ffmpeg_cmd_helper.set_input(input_files)
    ffmpeg_cmd_helper.set_output(file_format=file_format, out_stem="_resized", output_dir=output_dir)

    ffmpeg_cmd_helper.set_video_settings(encoding_speed=video_encoding_speed, quality=video_quality)
    ffmpeg_cmd_helper.set_bitrate(audio_bitrate=audio_bitrate, video_bitrate=video_bitrate)
    ffmpeg_cmd_helper.set_resolution_filter(resolution)

    ffmpeg_cmd_helper.execute()


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
]



# src\file_conversor\utils\bulma\form_output_dir.py

from typing import Any

from file_conversor.utils.dominate_bulma.form_input import FormFieldInput


def FormFieldOutputDirectory(
    help: str,
    _name: str,
    _readonly: bool = False,
    x_data: str = "",
    x_init: str = "",
    **kwargs,
):
    """
    Create a form field for selecting an output directory.

    :param help: Optional help text.
    :param _name: The name attribute for the select element.
    :param _readonly: Whether the input field is read-only.
    :param x_data: Additional x-data for the component.
    :param x_init: Additional x-init for the component.
    """
    return FormFieldInput(
        validation_expr="value.length > 0",
        _readonly=_readonly,
        _name=_name,
        help=help,
        button={
            "icon": {"name": "folder-open"},
            "_class": "ml-2 is-info",
            "_title": help,
            "@click": "openFolderDialog",
        },
        x_data=f"""
            async openFolderDialog() {{
                const folderList = await pywebview.api.open_folder_dialog({{ }});
                if (folderList && folderList.length > 0) {{
                    this.value = folderList[0];
                }}
            }},
            {x_data}
        """,
        x_init=f"""
            let thisObj = this;
            windowEventHandler.on('pywebviewready', async () => {{
                thisObj.value = await pywebview.api.get_last_open_dir();
            }});
            {x_init}
        """,
        **kwargs,
    )


__all__ = [
    "FormFieldOutputDirectory",
]



# src/file_conversor/backend/gui/pdf/index.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.gui._dom_page import home_nav_item
from file_conversor.backend.gui.pdf._dom_page import pdf_index_nav_item

from file_conversor.utils.dominate_bulma import PageCardGrid

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def pdf_index():
    tools = [
        {
            'image': url_for('icons', filename='padlock_locked.ico'),
            'title': _("Encrypt (protect PDF)"),
            'subtitle': _("Protect PDF file with a password (create encrypted PDF file)."),
            'url': url_for('pdf_encrypt'),
        },
        {
            'image': url_for('icons', filename='padlock_unlocked.ico'),
            'title': _("Decrypt (unprotect PDF)"),
            'subtitle': _("Remove password protection from a PDF file  (create decrypted PDF file)."),
            'url': url_for('pdf_decrypt'),
        },
        {
            'image': url_for('icons', filename='compress.ico'),
            'title': _("Compress"),
            'subtitle': _("Reduce the file size of a PDF document (requires Ghostscript external library)."),
            'url': url_for('pdf_compress'),
        },
        {
            'image': url_for('icons', filename='extract.ico'),
            'title': _("Extract pages"),
            'subtitle': _("Extract specific pages from a PDF document (create a new PDF file)."),
            'url': url_for('pdf_extract'),
        },
        {
            'image': url_for('icons', filename='merge.ico'),
            'title': _("Merge / Join PDFs"),
            'subtitle': _("Merge (join) input PDFs into a single PDF file."),
            'url': url_for('pdf_merge'),
        },
        {
            'image': url_for('icons', filename='rotate_right.ico'),
            'title': _("Rotate pages"),
            'subtitle': _("Rotate PDF pages (clockwise or anti-clockwise)."),
            'url': url_for('pdf_rotate'),
        },
        {
            'image': url_for('icons', filename='split.ico'),
            'title': _("Split PDF"),
            'subtitle': _("Split PDF pages into several 1-page PDFs."),
            'url': url_for('pdf_split'),
        },
        {
            'image': url_for('icons', filename='ocr.ico'),
            'title': _("OCR scanned PDF"),
            'subtitle': _("Create a searchable PDF file from scanned documents using OCR."),
            'url': url_for('pdf_ocr'),
        },
        {
            'image': url_for('icons', filename='convert.ico'),
            'title': _("Convert"),
            'subtitle': _("Convert a PDF file to a different format (might require LibreOffice)."),
            'url': url_for('pdf_convert'),
        },
        {
            'image': url_for('icons', filename='separate.ico'),
            'title': _("Extract images"),
            'subtitle': _("Extract images from a PDF document."),
            'url': url_for('pdf_extract_img'),
        },
        {
            'image': url_for('icons', filename='repair.ico'),
            'title': _("Repair PDF"),
            'subtitle': _("Attempt to repair a corrupted PDF file."),
            'url': url_for('pdf_repair'),
        },
    ]

    return render_template_string(str(
        PageCardGrid(
            *tools,
            nav_items=[
                home_nav_item(),
                pdf_index_nav_item(active=True),
            ],
            _title=f"{_('PDF Tools')} - File Conversor",
        )
    ))



# src\file_conversor\utils\validators.py

import math
import sys
import typer

from pathlib import Path
from typing import Any, Callable, Iterable, List

# user provided imports
from file_conversor.config.locale import get_translation

_ = get_translation()


def is_close(num1: int | float, num2: int | float, rel_tol: float = 1e-9, abs_tol: float = 1e-9) -> bool:
    """
    Determines if two numbers are close to each other within a specified tolerance.

    :param num1: First number.
    :param num2: Second number.
    :param rel_tol: Relative tolerance.
    :param abs_tol: Absolute tolerance.
    """
    return math.isclose(num1, num2, rel_tol=rel_tol, abs_tol=abs_tol)


def is_zero(num: int | float) -> bool:
    """
    Determines if a number is effectively zero within a small tolerance.

    :param num: The number to check.
    """
    return is_close(num, 0.0, rel_tol=1e-9, abs_tol=1e-9)


def prompt_retry_on_exception(
        text: str,
        default: Any | None = None,
        hide_input: bool = False,
        confirmation_prompt: bool | str = False,
        type: Any | None = None,
        show_choices: bool = True,
        show_default: bool = True,
        check_callback: Callable[[Any], Any] | None = None,
        retries: int | None = None,
        **prompt_kwargs,
) -> Any:
    """
    Prompts the user for input, retrying on exception.

    :param text: The prompt text.
    :param default: The default value.
    :param hide_input: Whether to hide the input (for passwords).
    :param confirmation_prompt: Whether to ask for confirmation.
    :param type: The type of the input.
    :param show_choices: Whether to show choices (for Enum types).
    :param show_default: Whether to show the default value.
    :param check_callback: A callback function to validate the input.
    :param retries: The number of retries (None for infinite).
    :param prompt_kwargs: Additional keyword arguments for typer.prompt.

    :raises typer.Abort: If the user aborts the input or retries are exhausted.
    :return: The user input, validated by the callback if provided.
    """
    for _ in range(retries or int(sys.maxsize)):
        try:
            if type == bool:
                res = typer.confirm(
                    text=text,
                    default=default if isinstance(default, bool) else False,
                    show_default=show_default,
                )
            else:
                res = typer.prompt(
                    text=text,
                    default=default,
                    hide_input=hide_input,
                    confirmation_prompt=confirmation_prompt,
                    type=type,
                    show_choices=show_choices,
                    show_default=show_default,
                    **prompt_kwargs,
                )
            return check_callback(res) if check_callback else res
        except:
            pass
    raise typer.Abort()


def check_file_size_format(data: str | None) -> str | None:
    exception = typer.BadParameter(f"{_('Invalid file size')} '{data}'. {_('Valid file size is <size>[K|M|G]')}.")
    if not data or data == "0":
        return data

    size_unit = data[-1].upper()
    if size_unit not in ["K", "M", "G"]:
        raise exception

    size_value = -1
    try:
        size_value = float(data[:-1])
    except ValueError:
        raise exception

    if size_value < 0:
        raise exception
    return data


def check_video_resolution(data: str | None) -> str | None:
    if not data:
        return data
    if ":" not in data:
        raise typer.BadParameter(f"{_('Invalid format for video resolution')} '{data}'. {_('Valid format is WIDTH:HEIGHT')}.")
    width_height = data.split(":")
    if len(width_height) != 2:
        raise typer.BadParameter(f"{_('Invalid format for video resolution')} '{data}'. {_('Valid format is WIDTH:HEIGHT')}.")
    return data


def check_path_exists(data: str | Path | None, exists: bool = True):
    if not data:
        return data
    path = Path(data)
    if exists and not path.exists():
        raise typer.BadParameter(f"{_("File")} '{path}' {_("not found")}")
    if not exists and path.exists():
        raise typer.BadParameter(f"{_("File")} '{path}' {_("exists")}")
    return data


def check_file_exists(data: str | Path | None):
    check_path_exists(data)
    if data and not Path(data).is_file():
        raise typer.BadParameter(f"{_("Path")} '{data}' {_("is not a file")}")
    return data


def check_dir_exists(data: str | Path | None, mkdir: bool = False):
    if data and mkdir:
        Path(data).mkdir(parents=True, exist_ok=True)
    check_path_exists(data)
    if data and not Path(data).is_dir():
        raise typer.BadParameter(f"{_("Path")} '{data}' {_("is not a directory")}")
    return data


def check_is_bool_or_none(data: str | bool | None) -> bool | None:
    """
    Checks if the provided input is a valid bool or None.
    """
    if data is None or isinstance(data, bool):
        return data
    if isinstance(data, str):
        if data.lower() == "true":
            return True
        if data.lower() == "false":
            return False
        if data.lower() == "none":
            return None
    raise typer.BadParameter(_("Must be a bool or None."))


def check_positive_integer(num: int | float | None, allow_zero: bool = False):
    """
    Checks if the provided number is a positive integer.
    """
    if num is None:
        return None
    if num < 0 or (not allow_zero and is_close(num, 0)):
        raise typer.BadParameter(_("Must be a positive integer."))
    return num


def check_file_format(filename_or_iter: list | dict | set | str | Path | None, format_dict: dict | list, exists: bool = False):
    """
    Checks if the provided format is supported.

    :param filename_or_iter: Filename or iterable list
    :param format_dict: Format {format:options} or [format]
    :param exists: Check if file exists. Default False (do not check).

    :raises typer.BadParameter: Unsupported format, or file not found.
    :raises TypeError: Invalid parameter type.
    """
    file_list = []
    if isinstance(filename_or_iter, (list, dict, set)):
        file_list = list(filename_or_iter)
    elif isinstance(filename_or_iter, (str | Path)):
        file_list.append(str(filename_or_iter))
    elif filename_or_iter is None:
        return filename_or_iter
    else:
        raise TypeError(f"{_('Invalid type')} '{type(filename_or_iter)}' {_('for')} filename_or_iter. {_('Valid values are Iterable | str | None')}.")
    for filename in file_list:
        file_path = Path(filename)
        file_format = file_path.suffix[1:]
        if format_dict and file_format not in format_dict:
            raise typer.BadParameter(f"\n{_('Unsupported format')} '{file_format}'. {_('Supported formats are')}: {', '.join([str(f) for f in format_dict])}.")
        if exists:
            check_file_exists(file_path)
    return filename_or_iter


def check_valid_options(data: Any | None, valid_options: Iterable):
    if not data:
        return data
    if data not in valid_options:
        raise typer.BadParameter(f"'{data}' {_('is invalid.  Valid options are')} {', '.join([str(v) for v in valid_options])}.")
    return data


def check_ip_format(data: str | None) -> str | None:
    exception = typer.BadParameter(f"'{data}' {_('is not a valid IP address')}.")
    if not data:
        return data
    parts = data.split(".")
    if len(parts) != 4:
        raise exception
    for part in parts:
        try:
            number = int(part)
            if number < 0 or number > 255:
                raise exception
        except ValueError:
            raise exception
    return data


__all__ = [
    "prompt_retry_on_exception",
    "check_file_size_format",
    "check_video_resolution",
    "check_path_exists",
    "check_file_exists",
    "check_dir_exists",
    "check_is_bool_or_none",
    "check_positive_integer",
    "check_file_format",
    "check_valid_options",
    "check_ip_format",
]



# src\file_conversor\utils\bulma_utils\__init__.py

from file_conversor.utils.bulma_utils.audio_video import *
from file_conversor.utils.bulma_utils.general import *
from file_conversor.utils.bulma_utils.image import *
from file_conversor.utils.bulma_utils.pdf import *



# src\file_conversor\cli\ppt\__init__.py

import typer

# user-provided modules
from file_conversor.config.locale import get_translation

from file_conversor.cli.ppt._typer import COMMAND_NAME

from file_conversor.cli.ppt.convert_cmd import typer_cmd as convert_cmd

_ = get_translation()

ppt_cmd = typer.Typer(
    name=COMMAND_NAME,
    help=f"{_('Presentation file manipulation')} {_('(requires LibreOffice)')})",
)
ppt_cmd.add_typer(convert_cmd)

__all__ = [
    "ppt_cmd",
]




# src\file_conversor\cli\pipeline\execute_cmd.py

import typer

from typing import Annotated

from rich import print

# user-provided modules
from file_conversor.backend import BatchBackend

from file_conversor.cli.pipeline._typer import COMMAND_NAME, EXECUTE_NAME

from file_conversor.config import Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils.rich_utils import get_progress_bar
from file_conversor.utils.validators import check_dir_exists

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = BatchBackend.EXTERNAL_DEPENDENCIES


@typer_cmd.command(
    name=EXECUTE_NAME,
    help=f"""
        {_('Execute file processing pipeline.')}        
    """,
    epilog=f"""
**{_('Examples')}:** 

- `file_conversor {COMMAND_NAME} {EXECUTE_NAME} c:/Users/Alice/Desktop/pipeline_name` 
""")
def execute(
    pipeline_folder: Annotated[str, typer.Argument(
        help=f"{_('Pipeline folder')}",
        callback=check_dir_exists
    )],
):
    logger.info("Executing pipeline ...")
    with get_progress_bar() as progress:
        batch_backend = BatchBackend(pipeline_folder)
        batch_backend.load_config()
        batch_backend.execute(progress)

    logger.info(f"{_('Pipeline execution')}: [bold green]{_('SUCCESS')}[/].")
    logger.info(f"--------------------------------")


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
]




# src\file_conversor\cli\pdf\encrypt_cmd.py

import typer

from pathlib import Path
from typing import Annotated, Any, Callable, Iterable, List

from rich import print

# user-provided modules
from file_conversor.backend.pdf import PyPDFBackend

from file_conversor.cli.pdf._typer import SECURITY_PANEL as RICH_HELP_PANEL
from file_conversor.cli.pdf._typer import COMMAND_NAME, ENCRYPT_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.typer_utils import InputFilesArgument, OutputDirOption
from file_conversor.utils.validators import check_valid_options

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = PyPDFBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    for ext in PyPDFBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="encrypt",
                description="Encrypt",
                command=f'cmd.exe /k "{Environment.get_executable()} "{COMMAND_NAME}" "{ENCRYPT_NAME}" "%1""',
                icon=str(icons_folder_path / "padlock_locked.ico"),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_pdf_encrypt_cmd(
    input_files: List[Path],

    decrypt_password: str | None,

    owner_password: str,
    user_password: str | None,

    permissions: Iterable[PyPDFBackend.EncryptionPermission] | None,

    encrypt_algo: str,
    output_dir: Path,
    progress_callback: Callable[[float], Any] = lambda p: p,
):
    pypdf_backend = PyPDFBackend(verbose=STATE["verbose"])

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        pypdf_backend.encrypt(
            # files
            input_file=input_file,
            output_file=output_file,

            # passwords
            owner_password=owner_password,
            user_password=user_password,
            decrypt_password=decrypt_password,

            # permissions
            permissions=permissions,

            encryption_algorithm=PyPDFBackend.EncryptionAlgorithm.from_str(encrypt_algo),
            progress_callback=lambda p: progress_callback(progress_mgr.update_progress(p))
        )
        progress_callback(progress_mgr.complete_step())

    cmd_mgr = CommandManager(input_files, output_dir=output_dir, overwrite=STATE["overwrite-output"])
    cmd_mgr.run(callback, out_stem="_encrypted")
    logger.info(f"{_('Encryption')}: [bold green]{_('SUCCESS')}[/].")


@typer_cmd.command(
    name=ENCRYPT_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Protect PDF file with a password (create encrypted PDF file).')}
        
        {_('Outputs a file with _encrypted at the end.')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor {COMMAND_NAME} {ENCRYPT_NAME} input_file.pdf -od D:/Downloads --owner-password 1234`

        - `file_conversor {COMMAND_NAME} {ENCRYPT_NAME} input_file.pdf -op 1234 --up 0000 -an -co`
    """)
def encrypt(
    input_files: Annotated[List[Path], InputFilesArgument(PyPDFBackend)],
    owner_password: Annotated[str, typer.Option("--owner-password", "-op",
                                                help=_("Owner password for encryption. Owner has ALL PERMISSIONS in the output PDF file."),
                                                prompt=f"{_('Owner password for encryption (password will not be displayed, for your safety)')}",
                                                hide_input=True,
                                                )],

    permissions: Annotated[List[PyPDFBackend.EncryptionPermission], typer.Option("--permission", "-p",
                                                                                 help=_("User permissions for the encrypted PDF file. Can be used multiple times to add multiple permissions. If no permissions are specified, the user will have no permissions (read-only)."),
                                                                                 )] = [PyPDFBackend.EncryptionPermission.NONE],

    user_password: Annotated[str | None, typer.Option("--user-password", "-up",
                                                      help=_("User password for encryption. User has ONLY THE PERMISSIONS specified in the arguments. Defaults to None (user and owner password are the same)."),
                                                      )] = None,

    decrypt_password: Annotated[str | None, typer.Option("--decrypt-password", "-dp",
                                                         help=_("Decrypt password used to open protected file. Defaults to None (do not decrypt)."),
                                                         )] = None,

    algorithm: Annotated[str, typer.Option("--algorithm", "-a",
                                           help=_("Encryption algorithm used. Valid options are RC4-40, RC4-128, AES-128, AES-256-R5, or AES-256. Defaults to AES-256 (for enhanced security and compatibility)."),
                                           callback=lambda x: check_valid_options(x, valid_options=PyPDFBackend.EncryptionAlgorithm.get_dict()),
                                           )] = PyPDFBackend.EncryptionAlgorithm.AES_256.value,

    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    execute_pdf_encrypt_cmd(
        input_files=input_files,
        decrypt_password=decrypt_password,
        owner_password=owner_password,
        user_password=user_password,
        permissions=permissions,
        encrypt_algo=algorithm,
        output_dir=output_dir,
    )


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
    "execute_pdf_encrypt_cmd",
]



# src\file_conversor\cli\win\_typer.py

# user-provided modules
from file_conversor.config import get_translation

_ = get_translation()

CONTEXT_MENU_PANEL = _("Context menu")
OTHERS_PANEL = _("Other commands")

# command
COMMAND_NAME = "win"

# SUBCOMMANDS
INSTALL_MENU_NAME = "install-menu"
UNINSTALL_MENU_NAME = "uninstall-menu"
RESTART_EXPLORER_NAME = "restart-explorer"

__all__ = [
    "COMMAND_NAME",

    "INSTALL_MENU_NAME",
    "UNINSTALL_MENU_NAME",
    "RESTART_EXPLORER_NAME",
]



# src\file_conversor\cli\win\__init__.py

import typer

# user-provided modules
from file_conversor.config.locale import get_translation

from file_conversor.cli.win._typer import COMMAND_NAME

from file_conversor.cli.win.install_menu_cmd import typer_cmd as install_menu_cmd
from file_conversor.cli.win.restart_explorer_cmd import typer_cmd as restart_explorer_cmd
from file_conversor.cli.win.uninstall_menu_cmd import typer_cmd as uninstall_menu_cmd

_ = get_translation()

win_cmd = typer.Typer(
    name=COMMAND_NAME,
    help=_("Windows OS commands (for Windows ONLY)"),
)
win_cmd.add_typer(restart_explorer_cmd)

# CONTEXT_MENU_PANEL
win_cmd.add_typer(install_menu_cmd)
win_cmd.add_typer(uninstall_menu_cmd)

__all__ = [
    "win_cmd",
]
